function [newCoordinates,normAxis,Theta] = RotRod2Vec(Vector1,Vector2,oldCoordinates)
%ROTROD2VEC Rotation Between 2 Vectors
%   [NEWCOORDINATES,NORMAXIS,THETA]=ROTROD2VEC(VECTOR1,VECTOR2,OLDCOORDINATES)
%   rotates given 3-D coordinates, OLDCOORDINATES, between to given 3-D vectors,
%   VECTOR1 and VECTOR2, and provides the normalized axis of rotation, NORMAXIS, the
%   rotation angle, THETA, as well as the resulting coordinates in NEWCOORDINATES.
%
%   INPUT:                                                                                    
%   VECTOR1 and VECTOR2 should be of size==[1 3] and their norms should be non-zero,
%   OLDCOORDINATES should be of size==[: 3].
%
%   OUTPUT:
%   NEWCOORDINATES is of the same size as OLDCOORDINATES. NORMAXIS is of the size==[1
%   3]. THETA is a real scalar.
%
%   SHORTCUT:
%   If cross(VECTOR1,VECTOR2)==0 the function returns to parent with
%   NEWCOORDINATES==OLDCOORDINATES
%
%   ADDITIONAL INFORMATION:                                                                                    
%   ROTROD2VEC is the Matlab analog to a VRML rotation between two 3-D vectors,
%   compare to the function block 'Rotation Between 2 Vectors', Library 'Virtual
%   Reality Toolbox' [1]. It rotates the given 3-D coordinates around the axis
%   perpendicular to the two input vectors with an angle equal to the angle between
%   the two input vectors. For this it calculates a rotation matrix according to
%   Rodrigues' Rotation Formula [2].
%
%   REFERENCES:
%   [1] Documentation for MathWorks Products, R2006b,  1994-2006 The MathWorks, Inc.
%   http://www.mathworks.com/access/helpdesk/help/toolbox/vr/index.html?/access/helpdesk/help/toolbox/vr/rotationmatrixtovrmlrotation.html
%   [2] Belongie, Serge. "Rodrigues' Rotation Formula." From MathWorld--A Wolfram Web
%   Resource, created by Eric W. Weisstein. http://mathworld.wolfram.com/RodriguesRotationFormula.html
%
%
%    2006 Lyonel Ehrl
% 
%   This program is free software; you can redistribute it and/or modify it under the
%   terms of the GNU General Public License as published by the Free Software
%   Foundation; either version 2 of the License, or (at your option) any later
%   version.
% 
%   This program is distributed in the hope that it will be useful, but WITHOUT ANY
%   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
%   PARTICULAR PURPOSE.  See the GNU General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License along with this
%   program; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
%   Suite 330, Boston, MA  02111-1307  USA
%
%   Any error reports or suggestion for improvement should be directed to
%   lyonel.ehrl@chem.ethz.ch.
%
%   First Version: 21 October 2006
%   Last update: 23 October 2006

    % Check input variables
        % (a) <Vector1>
        if exist('Vector1')==0,
            error('Input Variable <Vector1> not declared');
        elseif exist('Vector1')==1,
            if isnumeric(Vector1)==1,
                if size(Vector1)~=[1 3],
                    error('Input Variable <Vector1> has to be of size==[1 3]');
                elseif norm(Vector1)==0,
                    error('norm(Vector1)==0: Norm of Input Variable <Vector1> has to be non-zero');
                end
            else
                error('Input Variable <Vector1> has to be of Type <numeric> with size==[1 3]');
            end
        else
            error('Input <Vector1> has to be a Variable of Type <numeric> with size==[1 3]');
        end
        % (b) <Vector2>
        if exist('Vector2')==0,
            error('Input Variable <Vector2> not declared');
        elseif exist('Vector2')==1,
            if isnumeric(Vector2)==1,
                if size(Vector2)~=[1 3],
                    error('Input Variable <Vector2> has to be of size==[1 3]');
                elseif norm(Vector2)==0,
                    error('norm(Vector2)==0: Norm of Input Variable <Vector2> has to be non-zero');
                end
            else
                error('Input Variable <Vector2> has to be of Type <numeric> with size==[1 3]');
            end
        else
            error('Input <Vector2> has to be a Variable of Type <numeric> with size==[1 3]');
        end
        % (c) <oldCoordinates>
        if exist('oldCoordinates')==0,
            error('Input Variable <oldCoordinates> not declared');
        elseif exist('oldCoordinates')==1,
            if isnumeric(oldCoordinates)==1,
                if size(oldCoordinates,2)~=3,
                    error('Input Variable <oldCoordinates> has to be of size==[: 3]');
                end
            else
                error('Input Variable <oldCoordinates> has to be of Type <numeric> with size==[rows 3]');
            end
        else
            error('Input <oldCoordinates> has to be a Variable of Type <numeric> with size==[rows 3]');
        end
    
    % Calculate axis vector
    crossV1V2 = cross(Vector1,Vector2);
    % Calculate norm of axis vector
    normV1V2 = norm(crossV1V2);
    if normV1V2==0,
        %warning('norm(cross(Vector1,Vector2))==0, therefore newCoordinates=oldCoordinates');
        newCoordinates = oldCoordinates;
        normAxis = [];
        Theta = [];
        return
    else
        normAxis = crossV1V2/normV1V2;
    end
    
    % Calculate Angle between Vector1 and Vector2
    minusacos = -acos(dot(Vector1,Vector2)/norm(Vector1)/norm(Vector2));
    
    % Renaming Variables to better match Reference [2]
    Theta = minusacos;
    wx = normAxis(1);
    wy = normAxis(2);
    wz = normAxis(3);

    % Evaluate Some Multiple Intermediate Results
    cosTheta = cos(Theta);
    sinTheta = sin(Theta);
    
    % Calculate Rotation Matrix according to Rodrigues' Rotation Formula
    RotMatRod =  [   cosTheta+wx^2*(1-cosTheta)        wx*wy*(1-cosTheta)-wz*sinTheta   wy*sinTheta+wx*wz*(1-cosTheta);...
                     wz*sinTheta + wx*wy*(1-cosTheta)  cosTheta+wy^2*(1-cosTheta)      -wx*sinTheta+wy*wz*(1-cosTheta);...
                    -wy*sinTheta+wx*wz*(1-cosTheta)    wx*sinTheta+wy*wz*(1-cosTheta)   cosTheta+wz^2*(1-cosTheta)   ];
                
    % Rotate oldCoordinates with an Angle Theta around the Axis normAxis
    newCoordinates = oldCoordinates*RotMatRod;
end