## Copyright (C) 2007,2008,2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##
## usage: mufit
##

## -*- texinfo -*-
## @deftypefn{Function File} magfac2 (@var{spins_cell},@var{fstar},@var{Q},@var{flag_k0},@var{ffq})
## @deftypefnx{Function File} {[@var{mag_inter_vec},@var{fsq},@var{mag_vec_cart}]} = moment_fit
## (@var{spins_cell},@var{fstar},@var{Q},@var{flag_k0},@var{ffq})
## @*Calculate the magnetic interaction vector and the magnetic structure factor
##
## @*input:
##        @*1.- @var{spins_cell}: magnetic model
##        @*2.- @var{fstar}: list flag +/- magnetic satellite
##        @*3.- @var{Q}: list of [h,k,l]
##        @*4.- @var{flag_k0}: switch k_0 = -k_0 ?
##        @*5.- @var{ffq}: magnetic form factor
##
## @*output:
##        @*1.- @var{mag_inter_vec}: magnetic interaction vector
##        @*2.- @var{fsq}: structure factor^2
##        @*3.- @var{mag_vec_cart}]: component of structure factor
## @end deftypefn

function [mag_inter_vec,fsqr,mag_vec_cart]=magfac2(spins_cell,fstar,Q,flag_k0,ffq);
%vectorized 07.03.2009

%---- compute the fourier transform
switch flag_k0
case 0 %---- usual case
	nr_spins=spins_cell{1};
	A=[2:nr_spins+1];
	phase=fstar'*[[spins_cell{1,A}].phase];
	p=[[spins_cell{1,A}].spins_p];
	spinsp=(reshape(p,3,columns(p)/3))';
	q=[[spins_cell{1,A}].spins_q];
	spinsq=(reshape(q,3,columns(p)/3))';
	atpos=[[spins_cell{1,A}].ionpar];
	ionpar=(reshape(atpos,5,columns(atpos)/5))';
	B=ionpar(:,4);
	occ=ionpar(:,5);

	%---- fourier transform; occupation number; form factor; Debye-Waller factor
	QQ=dot(Q',Q')/(4*pi).^2;
	QQ=QQ';

	%---- occupation number: size nr_atoms x hkl
	occt=repmat(occ,1,rows(Q));
	%---- Fourier transform: size hkl x nr_atoms
	ftr=[exp(-I.*ionpar(:,1:3)*Q')'.*exp(-QQ*B').*occt'.*ffq];

	%---- Real and Imaginary parts: size hkl x nr_atoms
	phase_partA=ftr.*exp(-I.*2.*pi*phase);
	ffstar=repmat(fstar',1,nr_spins);
	phase_partB=phase_partA.*ffstar;

	%---- x component 
	P_part=phase_partA*spinsp(:,1);
	Q_part=phase_partB*spinsq(:,1);
	%---- the sum is done automatically
	Smagstrufac_x=P_part.+I*Q_part;

	%---- y component 
	P_part=phase_partA*spinsp(:,2);
	Q_part=phase_partB*spinsq(:,2);
	Smagstrufac_y=P_part.+I*Q_part;	

	%---- z component 
	P_part=phase_partA*spinsp(:,3);
	Q_part=phase_partB*spinsq(:,3);
	Smagstrufac_z=P_part.+I*Q_part;


case 1 %---- if k0 and -k0 produce Bragg peaks at same position
	nr_spins=spins_cell{1};
	A=[2:nr_spins+1];

	phase=abs(fstar)'*[[spins_cell{1,A}].phase];
	p=[[spins_cell{1,A}].spins_p];
	spinsp=(reshape(p,3,columns(p)/3))';
	q=[[spins_cell{1,A}].spins_q];
	spinsq=(reshape(q,3,columns(p)/3))';
	atpos=[[spins_cell{1,A}].ionpar];
	ionpar=(reshape(atpos,5,columns(atpos)/5))';
	B=ionpar(:,4);
	occ=ionpar(:,5);

	%---- fourier transform; occupation number; form factor; Debye-Waller factor
	QQ=dot(Q',Q')/(4*pi).^2;
	QQ=QQ';
	%---- occupation number: size nr_atoms x hkl
	occt=repmat(occ,1,rows(Q));

	%---- Fourier transform: size hkl x nr_atoms
	ftr=[exp(-I.*ionpar(:,1:3)*Q')'.*exp(-QQ*B').*occt'.*ffq];

	%---- x component 
	Smagstrufac_x=2.*sum((cos(2.*pi.*phase).*repmat(spinsp(:,1),1,rows(QQ))').*ftr.+(sin(2.*pi.*phase).*repmat(spinsq(:,1),1,rows(QQ))').*ftr,2);

	%---- y component 
	Smagstrufac_y=2.*sum((cos(2.*pi.*phase).*repmat(spinsp(:,2),1,rows(QQ))').*ftr.+(sin(2.*pi.*phase).*repmat(spinsq(:,2),1,rows(QQ))').*ftr,2);

	%---- z component 
	Smagstrufac_z=2.*sum((cos(2.*pi.*phase).*repmat(spinsp(:,3),1,rows(QQ))').*ftr.+(sin(2.*pi.*phase).*repmat(spinsq(:,3),1,rows(QQ))').*ftr,2);

endswitch

%------------------------------------------------------------------------------------------
%---- calculate magnetic interaction vector k x (M x k)
mag_vec_cart=0.2695.*[Smagstrufac_x Smagstrufac_y Smagstrufac_z];
Qnorm=dot(Q,Q,2).^0.5;
Qvec_norm=Q./repmat(Qnorm,1,columns(Q));
mag_inter_vec=cross(Qvec_norm,cross(mag_vec_cart,Qvec_norm,2),2);

%---- calculate the square of the magnetic structure factor
fsqr=mag_inter_vec(:,1).*conj(mag_inter_vec(:,1)).+...
     mag_inter_vec(:,2).*conj(mag_inter_vec(:,2)).+...
     mag_inter_vec(:,3).*conj(mag_inter_vec(:,3));

endfunction

