## Copyright (C) 2007,2008,2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##
## usage: mufit

## -*- texinfo -*-
## @deftypefn{Function File} moment (@var{lat_cell},@var{ion_cell},@var{moment_cell},@var{propvec_cell_rlu},@var{propvec_cell_AA},@var{spgsym_cell},@var{spgtrans_cell})
## @deftypefnx{Function File}
## {[@var{M_cell},@var{S_cart_sys_cell}]} = moment (@var{lat_cell},@var{ion_cell},@var{moment_cell},@var{propvec_cell_rlu},@var{propvec_cell_AA},@var{spgsym_cell},@var{spgtrans_cell})
## Calculate the magnetic moment configuration from the magnetic model in
## mag.inp
##
## @*input:
##      @*1.- the lattice parameters (@var{lat_cell})
##	@*2.- the atomic positions (@var{ion_cell})
##      @*3.- the magnetic model (@var{moment_cell})
##      @*4.- the propagation vector in rlu and AA^1 (@var{propvec_cell_rlu},@var{propvec_cell_AA})
##	@*5.- the magnetic symmetry operations (@var{spgsym_cell})
##	@*6.- the list of translation (@var{spgtrans_cell})
##
## @*output:
##	@*1.- the spins coordinates in the unit cell used to calculate the magnetic structure factor
## (@var{M_cell})
##	@*2.- the magnetic moment directions/size in different cells according to the
## list of translations (@var{S_cart_sys_cell})
## @end deftypefn

function [M_cell,S_cart_sys_cell]=moment(lat_cell,ion_cell,moment_cell,propvec_cell_rlu,propvec_cell_AA,spgsym_cell,spgtrans_cell);

as=lat_cell.ll(1);
bs=lat_cell.ll(2);
cs=lat_cell.ll(3);
aa=lat_cell.angl(1);
bb=lat_cell.angl(2);
cc=lat_cell.angl(3);
[Mcryst2cart,Mrec2cart]=lat2cart(as,bs,cs,aa,bb,cc);

%---- 1) transform from polar coordinates to xyz coordinates if necessary
%---- note that polar coordinates should be defined in a cartesian coordinate system in the input file
moment_cart_cell=cell(moment_cell{1}+1,1);
moment_cart_cell{1}=moment_cell{1};

if (moment_cell{columns(moment_cell)}.type  == 1)
	invMcryst2cart=Mcryst2cart^-1;
	for i=1:moment_cell{1}
		mom.fq=moment_cell{i+1}.fq;
		RM=moment_cell{i+1}.rm;
		IM=moment_cell{i+1}.im;
		rtheta=moment_cell{i+1}.rtheta/180*pi;
		rphi=moment_cell{i+1}.rphi/180*pi;
		itheta=moment_cell{i+1}.itheta/180*pi;
		iphi=moment_cell{i+1}.iphi/180*pi;
		rmz=RM.*cos(rtheta);
        	rmx=RM.*sin(rtheta)*cos(rphi);
        	rmy=RM.*sin(rtheta)*sin(rphi);
        	imz=IM.*cos(itheta);
        	imx=IM.*sin(itheta).*cos(iphi);
        	imy=IM.*sin(itheta).*sin(iphi);
		%
		%transform to crystal system to apply later symmetry operations
		%
        	rm_cart=invMcryst2cart*[rmx; rmy; rmz];
        	moment_cell{i+1}.mx=rm_cart(1);moment_cell{i+1}.my=rm_cart(2);moment_cell{i+1}.mz=rm_cart(3);
        	im_cart=invMcryst2cart*[imx; imy; imz];
        	moment_cell{i+1}.imx=im_cart(1);moment_cell{i+1}.imy=im_cart(2);moment_cell{i+1}.imz=im_cart(3);
	endfor
endif

mag_vec_cart=[0;0;0];
k0=[propvec_cell_AA{2}.kx propvec_cell_AA{2}.ky propvec_cell_AA{2}.kz];

%---- 1) calculate magnetic moments in the primitive cell to 
%---- be used for the structure factors
atoms_pos=zeros(1:3,1);
l=1;
%---- create list of ion/magnetic moment according to symmetry elements
spins_cell=cell(spgsym_cell{1}*ion_cell{1}+1,1);
spins_cell{1}=ion_cell{1}*spgsym_cell{1};

while (l<=spgsym_cell{1}*ion_cell{1})
	for k=1:ion_cell{1}
        	X=ion_cell{k+1}.x;
        	Y=ion_cell{k+1}.y;
        	Z=ion_cell{k+1}.z;
        	for j=1:spgsym_cell{1}
%---- apply symmetry operations to atomic positions and transform to cartesian coordinates
			atoms_pos(1:3)=Mcryst2cart*([eval(spgsym_cell{l+1}.x).*as eval(spgsym_cell{l+1}.y).*bs eval(spgsym_cell{l+1}.z).*cs])';

%---- apply symmetry operations to magnetic moments
			RMx=moment_cell{k+1}.mx;
			RMy=moment_cell{k+1}.my;
			RMz=moment_cell{k+1}.mz;
			IMx=moment_cell{k+1}.imx;
			IMy=moment_cell{k+1}.imy;
			IMz=moment_cell{k+1}.imz;

			%%propagate the magnetic moment with the propagation vector and translation part of symmetry operations
			a1=spgsym_cell{l+1}.x;a2=spgsym_cell{l+1}.y;a3=spgsym_cell{l+1}.z;
			b1=a1(max([rindex(a1,"X"),rindex(a1,"Y"),rindex(a1,"Z")])+1:length(a1));
			b2=a2(max([rindex(a2,"X"),rindex(a2,"Y"),rindex(a2,"Z")])+1:length(a2));
			b3=a3(max([rindex(a3,"X"),rindex(a3,"Y"),rindex(a3,"Z")])+1:length(a3));
			if (length(b1)==0)
				b1='0';
			endif
			if (length(b2)==0)
				b2='0';
			endif
			if (length(b3)==0)
				b3='0';
			endif
			trans=exp(I*k0*Mcryst2cart*[eval(b1)*as;eval(b2)*bs;eval(b3)*cs]);
			atoms(l,1:11)=[atoms_pos ion_cell{k+1}.B ion_cell{k+1}.occ...
				eval(spgsym_cell{l+1}.rmx).*trans eval(spgsym_cell{l+1}.rmy).*trans eval(spgsym_cell{l+1}.rmz).*trans...
				eval(spgsym_cell{l+1}.imx).*trans eval(spgsym_cell{l+1}.imy).*trans eval(spgsym_cell{l+1}.imz).*trans];
			MM.ion=ion_cell{k+1}.ion;
                	MM.fq=moment_cell{k+1}.fq;
                	MM.ionpar=atoms(l,1:5);
                	MM.spins_p=real(atoms(l,6:8))./2;
                	MM.spins_q=real(atoms(l,9:11))./2;
			%the phase is the sum of two components, nth(spgsym,l+1).phase is given by symmetry
                	MM.phase=moment_cell{k+1}.phase.+spgsym_cell{l+1}.phase;
			MM.radius=ion_cell{k+1}.radius;
			MM.red=ion_cell{k+1}.red;
			MM.blue=ion_cell{k+1}.blue;
			MM.green=ion_cell{k+1}.green;
                	spins_cell{l+1}=MM;
                	l++;
		endfor
	endfor
endwhile

%---- 2) calculate the magnetic moments not only in the primitive cell but
%---- also according to the translations given in the input file
%M_cell=cell(2.*rows(atoms)+1,1);
M_cell=cell(rows(atoms)+1,1);
M_cell{1}=(spgtrans_cell{1}+1)*rows(atoms);
for l=1:rows(atoms)
        M.label=spins_cell{l+1}.ion;
        M.fq=spins_cell{l+1}.fq;
        M.mat(1:3)=atoms(l,1:3);
        M.T=[0;0;0];
	M.radius=spins_cell{l+1}.radius;
	M.red=spins_cell{l+1}.red;
	M.blue=spins_cell{l+1}.blue;
	M.green=spins_cell{l+1}.green;

%---- it is cpu-cheaper to put content of cell first in a variable
	sphase=2.*pi.*spins_cell{l+1}.phase;

        M.mpx=atoms(l,6).*cos(sphase);
        M.mpy=atoms(l,7).*cos(sphase);
        M.mpz=atoms(l,8).*cos(sphase);
        M.mqx=atoms(l,9).*sin(sphase);
        M.mqy=atoms(l,10).*sin(sphase);
        M.mqz=atoms(l,11).*sin(sphase);
        M_cell{l+1}=M;
endfor

kk=rows(atoms)+2;
for i=1:spgtrans_cell{1}
	Tx=spgtrans_cell{i+1}.x;Ty= spgtrans_cell{i+1}.y;Tz=spgtrans_cell{i+1}.z;
	T=(Mcryst2cart*[Tx;Ty;Tz])';
        for l=1:rows(atoms)
                M.label=spins_cell{l+1}.ion;
                M.fq=spins_cell{l+1}.fq;
                M.mat(1:3)=atoms(l,1:3).+T;
                M.T=T;
		M.radius=spins_cell{l+1}.radius;
		M.red=spins_cell{l+1}.red;
		M.blue=spins_cell{l+1}.blue;
		M.green=spins_cell{l+1}.green;
		sphase=T*k0'.+2.*pi.*spins_cell{l+1}.phase;
                M.mpx=atoms(l,6).*cos(sphase);
                M.mpy=atoms(l,7).*cos(sphase);
                M.mpz=atoms(l,8).*cos(sphase);
                M.mqx=atoms(l,9).*sin(sphase);
                M.mqy=atoms(l,10).*sin(sphase);
                M.mqz=atoms(l,11).*sin(sphase);
                M_cell{kk}=M;
		kk=kk+1;
        endfor
endfor

%---- 3)transform spins to cartesian coordinates for calculation of structure factor
S_cart_cell=cell(1,spins_cell{1}+1);
S_cart_sys_cell{1}=spins_cell{1};
atoms=zeros(spgsym_cell{1}*ion_cell{1},11);
for k=1:spins_cell{1}
	p.fq=spins_cell{k+1}.fq;
	p.ion=spins_cell{k+1}.ion;
	p.ionpar=spins_cell{k+1}.ionpar;

	p1=spins_cell{k+1}.spins_p(1);p2=spins_cell{k+1}.spins_p(2);p3=spins_cell{k+1}.spins_p(3);
	q1=spins_cell{k+1}.spins_q(1);q2=spins_cell{k+1}.spins_q(2);q3=spins_cell{k+1}.spins_q(3);
	if (moment_cell{columns(moment_cell)}.type  == 1)
		%magnetic moments in polar coordinates are already defined in a cartesian system 
		p.spins_p=[p1;p2;p3]';
		p.spins_q=[q1;q2;q3]';
	else
		%otherwise transform to cartesian system
		p.spins_p=(Mcryst2cart*[p1;p2;p3])';
		p.spins_q=(Mcryst2cart*[q1;q2;q3])';
	endif
	p.phase=spins_cell{k+1}.phase;
 	S_cart_sys_cell{k+1}=p;
endfor

endfunction

