## Copyright (C) 2007,2008,2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##
## usage: mufit
##
## Author: Bertrand Roessli <bertrand.roessli@psi.ch>

## -*- texinfo -*-
## @deftypefn{Function File}  mufit
## @deftypefnx{Function File} mufit
## 
## @setfilename mufit.info 
## @settitle Mufit Manual:
## @setchapternewpage odd
## @c
##
## @node    Top,       First Chapter, ,         (dir)
## @comment node-name, next,          previous, up   
##
## @menu
## * First Chapter::
## * Second Chapter:: 
## @end menu
##
## @node     First Chapter, Second Chapter, Top,      Top
## @comment  node-name,     next,          previous,  up
##
## @chapter First Chapter
## @cindex Introduction
##
## Mufit calculates or fits neutron nuclear/magnetic intensities or neutron polarisation 
## matrices. The program has been tested with Octave-3.2.0. 
##
## @itemize @bullet
## @item 
## The 'Mufit' function  does not take any argument but 
## uses 3 input files: 1- 'nuc.inp'; 2- 'crys.inp'; 3- 'mag.inp':
##
## @item
## The nuclear model is given in the file 'nuc.inp';
## The magnetic model is given in the file 'mag.inp';
## To fit polarimetry data, the file 'nuc.inp' is required in case 
## of nuclear-magnetic interaction terms.
##
## @item
## Lists of HKL/Intensities/Standard deviations are given in
## the files 'nuc.inp' and 'crys.inp'. The polarisation matrices are 
## given in the file 'mag.inp'.
##
## @item
## To calculate the neutron polarisation matrices 
## it is necessary to give a second vector in the scattering plane 
## to define the crystal orientation.
## (see mag.inp for an example)
##
## @item
## The coefficients of the magnetic formfactor are given in the 
## function 'formfac.m'. You will probably need to add the coefficients 
## for your case, as currently only a few examples are implemented. 
##
## @item
## The data can be fitted with the Levenberg-Marquard method or 
## the Nelder-Mead algorithm. Please note that the size of the initial
## Simplex depends on the number of parameters. Therefore it is recommended
## that if the Simplex method is used the number of cycles increases with
## increasing number of parameters (~1000 cycles for 20 parameters)
##
## @item
## Simulated Annealing has also been implemented. The algorithm is 
## described in the function samin.cc.
##
## @item  
## The fit functions have been adapted from the 'optim-1.0.6' package at
## www.octaveforge.org 
##
## @item
## The fit options should be given in the files 'nuc.inp' and 'mag.inp'. 
## See leasrq.m, nmsmax.m and samin.cc for details.
## Documentation can be obtained by typing "help function_name" at the Octave prompt.
##
## @item
## To install mufit you will need to compile __polmat.cc and samin.cc. 
## At the Octave prompt, type mkoctfile ***.cc
## 
## @item
## If you use the standalone application, just run the mufit executable.
## The scripts, cpp functions and input files should be in the same directory than 
## the executable. Please note that Octave needs to be installed even if 
## you prefer to use the executable.
##
## @item
## Please report bugs to bertrand.roessli_at_psi.ch
##
## @end itemize
## @node     Second Chapter, First Chapter,      Top
## @chapter Second Chapter
## @printindex cp
## @contents
## @seealso{Mufit manual}
## @end deftypefn

function  exit = mufit(option)
%----  main function to calculate/fit neutron polarisation matrices with a few other options

warning("off","all");
exit=1;
more;

disp(' ');
if (nargin == 0)
  cont=menu('--> Mufit-4.0 (bertrand.roessli@psi.ch), available options:',
	  'calculate the magnetic moment configuration',
          'calculate the magnetic interaction vectors/ the structure factors',
	  'calculate the single-crystal magnetic intensities',
	  'calculate the polarisation matrices using the Blume''s equations',
	  'calculate the polarisation matrices/structure factors with domains',
          'fit the single-crystal nuclear intensities',
	  'fit the single-crystal magnetic intensities',
	  'fit the polarisation matrices',
	  'draw the magnetic structure (balls)',
	  'draw the nuclear structure (balls/thermal ellipsoids)',
	  'space group symmetry operations',
          'calculate equivalent B-factors and displacement',
          'simulate flipping ratios (dipole approximation)',
	  'doc',
	  'exit');
elseif (nargin == 1)
  cont=option;
else
  usage('mufit(option)'); 
  print_usage(); 
endif




if (cont == 1)
%---- calculate the magnetic configuration and print out a list with spin directions
%---- useful to check magnetic structure against input file
%---- calls: lat2cat.m,moment.m

        s=input("crys.inp (0) or mag.inp (1)?: ");
        switch s
          case 0
            [genpar,singlecrys,...
            as,bs,cs,aa,bb,cc,...
            param,dp,...
            flag_k0,flag_nmi,k0,moment_type,...
            fit_type,simplex_options,lmfit_options,sim_ann_par,...
            lat_cell,ion_cell,moment_cell,propvec_cell_rlu,propvec_cell_AA,spgsym_cell,spgtrans_cell,domains_cell,...
            draw_mag_opt]=...
            crys_inp;
          case 1
            [as,bs,cs,aa,bb,cc,param,dp,flag_k0,flag_nmi,kk0,hhkl_AA,hhkl_rlu,AA1,Pi,Pfo,dPfo,nr,P_1,P_2,moment_type,...
            fit_type,simplex_options,lmfit_options,sim_ann_par,...
            lat_cell,ion_cell,moment_cell,propvec_cell_rlu,propvec_cell_AA,spgsym_cell,spgtrans_cell,domains_cell,...
            draw_mag_opt]=readmu;
        endswitch

        [Mcryst2cart,Mrec2cart]=lat2cart(as,bs,cs,aa,bb,cc);
tic
        [M_cell,spins_cart_cell]=moment(lat_cell,ion_cell,moment_cell,propvec_cell_AA,spgsym_cell,spgtrans_cell);
toc
	for k=1:M_cell{1}
                at_rec_units=Mcryst2cart^-1*M_cell{k+1}.mat';
                at_rec_units=[at_rec_units(1)./as,at_rec_units(2)./bs,at_rec_units(3)./cs];
		MM_p=Mcryst2cart*[M_cell{k+1}.mpx;M_cell{k+1}.mpy;M_cell{k+1}.mpz];
		MM_q=Mcryst2cart*[M_cell{k+1}.mqx;M_cell{k+1}.mqy;M_cell{k+1}.mqz];
		MM=sqrt(dot(MM_p,MM_p).+dot(MM_q,MM_q));
                %MM=sqrt(norm([M_cell{k}.mpx;M_cell{k}.mpy;M_cell{k}.mpz])^2+norm([M_cell{k}.mqx;M_cell{k}.mqy;M_cell{k}.mqz])^2);
                fprintf(stdout,'%s: (%+2.2f, %+2.2f, %+2.2f) | (%+2.2f, %+2.2f, %+2.2f)| (%+2.2f, %+2.2f, %+2.2f) | (%+2.2f, %+2.2f, %+2.2f) | %3.2f \n',...
                                M_cell{k+1}.label,M_cell{k+1}.T,at_rec_units,...
                                M_cell{k+1}.mpx,M_cell{k+1}.mpy,M_cell{k+1}.mpz,...
                                M_cell{k+1}.mqx,M_cell{k+1}.mqy,M_cell{k+1}.mqz,...
                                MM);
	endfor
        fflush(stdout);

elseif (cont == 2)
%---- calculate the magnetic interaction vector and structure factor
%---- the magnetic interaction vector is printed out in both crystal and mupad coordinate systems.
%---- calls: lat2cart.m,moment.m, magfac.m

	[as,bs,cs,aa,bb,cc,param,dp,flag_k0,flag_nmi,kk0,hhkl_AA,hhkl_rlu,AA1,Pi,Pfo,dPfo,nr,P_1,P_2,moment_type,...
          fit_type,simplex_options,lmfit_options,sim_ann_par,...
          lat_cell,ion_cell,moment_cell,propvec_cell_rlu,propvec_cell_AA,spgsym_cell,spgtrans_cell,domains_cell,...
          draw_mag_opt]=readmu;

tic
	fprintf( stdout,"\nSat.    H      K      L           Magnetic interaction vector in crystal       /        MuPAD coordinate systems                        F_mag^2 \n");
	fprintf(stdout,"---------------------------------------------------------------------------------------------------------------------------------------------------\n");

        [Mcryst2cart,Mrec2cart]=lat2cart(as,bs,cs,aa,bb,cc);
        [M_cell,spins_cart_cell]=moment(lat_cell,ion_cell,moment_cell,propvec_cell_AA,spgsym_cell,spgtrans_cell);
	ll=1;
	for kk=1:6:nr  
        	for ii=1:domains_cell{1}
		%---- rotation matrices for domains are given in a cartesian coordinates in input file
                	hkl_all_rlu(1:3,ll)=(Mcryst2cart^-1*domains_cell{ii+1}.rot*Mcryst2cart)*(hhkl_rlu(kk,1:3))';
                	hkl_all(1:3,ll)=domains_cell{ii+1}.rot*(hhkl_AA(kk,1:3))';
                	AA1_all(1:3,ll)=domains_cell{ii+1}.rot*AA1(kk,1:3)';
                	det_r(ll)=domains_cell{ii+1}.det;
                	dtype(ll)=domains_cell{ii+1}.type;
                	Pi_all(1:3,ll)=Pi(kk,1:3)';
                	popul(ll)=domains_cell{ii+1}.pop;
                	ll++;
        	endfor
	endfor

	%---- loop over hkl and calculates polarisation
	for k=1:ll-1
        	%Q-vector
        	H=hkl_all(1,k);
        	K=hkl_all(2,k);
        	L=hkl_all(3,k);
        	%----- Q vector in cartesian coordinates (1/A)
                Q=[H;K;L]';

		%---- mag. inter. vec., |F|^2 and magnetic interaction vector in cartesian coordinates
		[mag_inter_vec,fsqr,mag_vec_cart,fstar]=magfac(spins_cart_cell,Q,hkl_all_rlu(:,k)',propvec_cell_rlu,flag_k0,dtype(k));

		%---- transform magnetic interaction vector into Cryopad/Mupad coordinate system
		V1=[H;K;L];
		%---- component of vector in scattering plane in cartesian coordinates (1/A)
		V2=AA1_all(:,k);

		if (dot(V1/norm(V1),V2/norm(V2)) == 1 )
		        error("\nredefine scattering plane\n");
		        return;
		endif

		%----- Form unit vectors V1, V2 in scattering plane V3 up
		V3=cross(V1,V2);
		V2=cross(V3,V1);
		V3=V3/sqrt(sum(V3.*V3));
		V2=V2/sqrt(sum(V2.*V2));
		V1=V1/sqrt(sum(V1.*V1));
		V3=V3.*det_r(k);
		%----- coordinates of HKL in V1, V2, V3 frame
		%----- transformation matrix
		U=[V1';V2';V3'];
		%----- rotate magnetic interaction vector into MuPAD system
		mag_inter_vec_mu=U*mag_inter_vec;
                bragg=Mrec2cart^-1*[H;K;L];

		%----- print results
                if (fstar == -1) 
                  sat = ' - ';
                else 
                  sat =' + ';
                endif
		fprintf(stdout,"%s  %+2.3f %+2.3f %+2.3f | (%+2.3f %+2.3f*I, %+2.3f %+2.3f*I, %+2.3f %+2.3f*I) | (%+2.3f %+2.3f*I, %+2.3f %+2.3f*I, %+2.3f %+2.3f*I) | %2.3f\n",...
		sat,...
		bragg(1),bragg(2),bragg(3),...
		real(mag_inter_vec(1)),imag(mag_inter_vec(1)),...
		real(mag_inter_vec(2)),imag(mag_inter_vec(2)),...
		real(mag_inter_vec(3)),imag(mag_inter_vec(3)),...
		real(mag_inter_vec_mu(1)),imag(mag_inter_vec_mu(1)),...
		real(mag_inter_vec_mu(2)),imag(mag_inter_vec_mu(2)),...
		real(mag_inter_vec_mu(3)),imag(mag_inter_vec_mu(3)),...
		fsqr);
	endfor
toc

elseif (cont == 3)
%-----calculate magnetic single crystal intensities with domains

tic;

%---- get input from file "crys.inp" (see crys_inp.m);
	  [genpar,singlecrys,...
          as,bs,cs,aa,bb,cc,...
          param,dp,...
          flag_k0,flag_nmi,k0,moment_type,...
          fit_type,simplex_options,lmfit_options,sim_ann_par,...
          lat_cell,ion_cell,moment_cell,propvec_cell_rlu,propvec_cell_AA,spgsym_cell,spgtrans_cell,domains_cell,...
          draw_mag_opt]=...
          crys_inp;


%----- get the list of magnetic moments in the unit cell in cartesian coordinates 
	[M_cell,spins_cart_cell]=moment(lat_cell,ion_cell,moment_cell,propvec_cell_AA,spgsym_cell,spgtrans_cell);

	lambda=genpar.lambda;
%---- prepare hkl-list with domains, calculate formfactor, Lorenz factor 
domains_cell
	[QQ,fstar,formfactor,popul,llor]=hklprep(as,bs,cs,aa,bb,cc,lambda,singlecrys,domains_cell,spins_cart_cell,propvec_cell_rlu);

%----- get magnetic structure factor for the hkl-list with domains
	[mag_inter_vec,fsqr,mag_vec_cart]=magfac2(spins_cart_cell,fstar,QQ,flag_k0,formfactor);

%----- calculate extinction
	[extinc]=extinc(fsqr',genpar.lambda,genpar.zach,QQ);

%----- can proceed with calculations of the intensities
%----- magnetic intensity for each hkl
	if (singlecrys{1}.intfsqr == 0) %---- true if integrated intensities not corrected for Lorentz factor
		I_hkl=popul.*fsqr'.*llor.*extinc;
	else
		I_hkl=popul.*fsqr'.*extinc;
	endif
%----- reshape to sum over the magnetic domains
	I_hkl_dom=reshape(I_hkl,domains_cell{1},columns(I_hkl)/domains_cell{1})';
%----- calculated magnetic intensities averaged over the domains
	if (domains_cell{1} > 1)
		I_crys=sum(I_hkl_dom,2);
	else
		I_crys=I_hkl_dom;
	endif
%----- print out calculations
	nr_hkl=singlecrys{2};
	hkl_crys_rlu=[[singlecrys{1,3:nr_hkl.+2}].hhkl_rlu];
	fprintf(stdout,'H    K    L    Int       Lorenz factor\n');
	for k=1:nr_hkl
		hkl_print=[[singlecrys{1,k+2}].hhkl_rlu];
		fprintf(stdout,'%+f %+f %+f      %f   %f  \n',hkl_print(1),hkl_print(2),hkl_print(3),I_crys(k),llor(k*domains_cell{1}));
	endfor
toc

elseif (cont == 4)
%---- calculate the polarisation matrices according to Blume's formula

%---- import everything necessary to calculate and fit polarimetry data, see readmu.m
	[as,bs,cs,aa,bb,cc,param,dp,flag_k0,flag_nmi,kk0,hhkl_AA,hhkl_rlu,AA1,Pi,Pfo,dPfo,nr,P_1,P_2,moment_type,...
	fit_type,simplex_options,lmfit_options,sim_ann_par,...
	lat_cell,ion_cell,moment_cell,propvec_cell_rlu,propvec_cell_AA,spgsym_cell,spgtrans_cell,domains_cell,...
	draw_mag_opt]=readmu;

tic;
	[pf]=pol(reshape(Pi',1,3*nr),hhkl_AA,hhkl_rlu,AA1,nr,flag_k0,flag_nmi,...
		lat_cell,ion_cell,spgsym_cell,spgtrans_cell,propvec_cell_rlu,propvec_cell_AA,moment_cell);
	pol_calc=(reshape(pf,3,nr))';

	%----- Print out a few information before starting calculations
	fprintf(stdout,"\nPolarisation matrices calculated from Blume's equations\n");
	fprintf(stdout,"! domains are NOT taken into account !\n");
	fprintf(stdout,"! efficiency of polariser/analyser assumed to be 1 !\n\n");
	fprintf(stdout," H       K       L  ||  Pol. (incident) |   Pol. (measured)|   Pol. (calculated)\n");
	
	%----- transform to cartesian system
	[Mcryst2cart,Mrec2cart]=lat2cart(as,bs,cs,aa,bb,cc);
        bragg_ind=(Mrec2cart^-1*hhkl_AA')';
        fprintf(stdout,'%+2.3f %+2.3f %+2.3f | %+2.2f %+2.2f %+2.2f|  %+2.2f %+2.2f %+2.2f | %+2.2f %+2.2f %+2.2f\n',[bragg_ind,Pi,Pfo,pol_calc]');

	fprintf(stdout,'\nelapsed time %+3.2f (s)\n',toc);


elseif (cont == 5)
%---- Calculate polarisation matrices and structure factors^2 with bender efficiencies 
%---- The polarisation matrices and cross sections are calculated from the neutron cross-section

%---- import everything necessary to calculate and fit polarimetry data, see readmu.m
	[as,bs,cs,aa,bb,cc,param,dp,flag_k0,flag_nmi,kk0,hhkl_AA,hhkl_rlu,AA1,Pi,Pfo,dPfo,nr,P_1,P_2,moment_type,...
	fit_type,simplex_options,lmfit_options,sim_ann_par,...
	lat_cell,ion_cell,moment_cell,propvec_cell_rlu,propvec_cell_AA,spgsym_cell,spgtrans_cell,domains_cell,...
	draw_mag_opt]=readmu;

	[matrix]=fopen('matrix.lis','wt');
	fprintf(stdout,'-->Polarisation matrices and cross-sections for Bender efficiencies of P_1= %+3.2f and P_2= %+3.2f\n\n',P_1,P_2);
	fprintf(matrix,'-->Polarisation matrices and cross-sections for Bender efficiencies of P_1= %+3.2f and P_2= %+3.2f\n\n',P_1,P_2);

%----- transformation matrix to cartesian system for real and reciprocal space
	[Mcryst2cart,Mrec2cart]=lat2cart(as,bs,cs,aa,bb,cc);
	[M_cell,spins_cart_cell]=moment(lat_cell,ion_cell,moment_cell,propvec_cell_AA,spgsym_cell,spgtrans_cell);

ll=1;
if (flag_nmi ~= 0)
	[genpar_n,as_n,bs_n,cs_n,aa_n,bb_n,cc_n,ion_cell_n,spgsym_cell_n,spgtrans_cell_n,twins_n,singlecrys_n,...
		parameters_n,dparameters_n,...
		nfit_type,simplexnuc_options,lmfitnuc_options,nsim_ann_par,...
		draw_nuc_opt]=readnuc;
endif


for kk=1:3:nr
	if (flag_nmi ~= 0)
		[nf]=feval("nucfac",as_n,bs_n,cs_n,aa_n,bb_n,cc_n,ion_cell_n,spgsym_cell_n,spgtrans_cell_n,hhkl_AA(kk,1:3));		
        else
                nf=0;
	endif
        for ii=1:domains_cell{1}
%---- rotation matrices for domains are given in a cartesian coordinates in input file
                hkl_all_rlu(1:3,ll)=(Mcryst2cart^-1*domains_cell{ii+1}.rot*Mcryst2cart)*(hhkl_rlu(kk,1:3))';
                hkl_all(1:3,ll)=domains_cell{ii+1}.rot*(hhkl_AA(kk,1:3))';
                AA1_all(1:3,ll)=domains_cell{ii+1}.rot*AA1(kk,1:3)';
                Pi_all(1:3,ll)=Pi(kk,1:3)';
                det_r(ll)=domains_cell{ii+1}.det;
                dtype(ll)=domains_cell{ii+1}.type;
                popul(ll)=domains_cell{ii+1}.pop; 
                nucstrucfac(ll)=nf;
                ll++;
        endfor
endfor
tic;
jj=1;
for k=1:domains_cell{1}:ll-1
	fprintf(stdout,"                                                               Final Polarisation                                                      Intensity (+ :--> +)                                                     Intensity (+ :--> -)\n");
	fprintf(stdout,"Type     Popul. |   Q (rlu)            |  XX     XY    XZ   |   YX    YY    YZ     |   ZX    ZY    ZZ         |  XX     XY    XZ   |   YX    YY    YZ     |   ZX    ZY    ZZ         |  X-X    X-Y   X-Z  |   Y-X   Y-Y   Y-Z     |   Z-X   Z-Y   Z-Z    |\n");
	fprintf(matrix,"                                                               Final Polarisation                                                      Intensity (+ :--> +)                                                     Intensity (+ :--> -)\n");
	fprintf(matrix,"Type     Popul. |   Q (rlu)            |  XX     XY    XZ   |   YX    YY    YZ     |   ZX    ZY    ZZ         |  XX     XY    XZ   |   YX    YY    YZ     |   ZX    ZY    ZZ         |  X-X    X-Y   X-Z  |   Y-X   Y-Y   Y-Z     |   Z-X   Z-Y   Z-Z    |\n");
%---- Pi along x
        I_pp=zeros(3,3);
        I_pm=zeros(3,3);
        polar=zeros(3,3);
        for ii=1:domains_cell{1}
                H=hkl_all(1,jj);
                K=hkl_all(2,jj);
                L=hkl_all(3,jj);
                %----- Q vector in cartesian coordinates (1/A)
                Q=[H;K;L]';   
                Q_rlu=[hkl_all_rlu(1,jj);hkl_all_rlu(2,jj);hkl_all_rlu(3,jj)]';
                if (ii == 1) Q_rlu_print=Q_rlu; endif;
                %---- second vector in scattering plane
                %---- first vector is given by Q
                A1=AA1_all(1:3,jj);
                [I_pp_1d,I_pm_1d,polar_1d]=...
                feval("onedomain",P_1,P_2,Q,Q_rlu,A1',flag_k0,flag_nmi,spins_cart_cell,dtype(jj),'pi_up',propvec_cell_rlu,det_r(jj),nucstrucfac(jj));

                fprintf(stdout,'%s       %+3.2f     %+3.2f %+3.2f %+3.2f      %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f         %+3.2f %+3.2f %+3.2f      %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f         %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f\n',...
                        dtype(jj),popul(jj),...
                        Q_rlu(1),Q_rlu(2),Q_rlu(3),...
                        polar_1d(1,1),polar_1d(1,2),polar_1d(1,3),polar_1d(2,1),polar_1d(2,2),polar_1d(2,3),polar_1d(3,1),polar_1d(3,2),polar_1d(3,3),...
                        real(I_pp_1d(1,1)), real(I_pp_1d(1,2)),real(I_pp_1d(1,3)),...
                        real(I_pp_1d(2,1)), real(I_pp_1d(2,2)),real(I_pp_1d(2,3)),...
                        real(I_pp_1d(3,1)), real(I_pp_1d(3,2)),real(I_pp_1d(3,3)),...
                        real(I_pm_1d(1,1)), real(I_pm_1d(1,2)),real(I_pm_1d(1,3)),...
                        real(I_pm_1d(2,1)), real(I_pm_1d(2,2)),real(I_pm_1d(2,3)),...
                        real(I_pm_1d(3,1)), real(I_pm_1d(3,2)),real(I_pm_1d(3,3)));  
                fprintf(matrix,'%s       %+3.2f     %+3.2f %+3.2f %+3.2f      %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f       %+3.2f %+3.2f %+3.2f      %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f         %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f\n',...
                        dtype(jj),popul(jj),...
                        Q_rlu(1),Q_rlu(2),Q_rlu(3),...
                        polar_1d(1,1),polar_1d(1,2),polar_1d(1,3),polar_1d(2,1),polar_1d(2,2),polar_1d(2,3),polar_1d(3,1),polar_1d(3,2),polar_1d(3,3),...
                        real(I_pp_1d(1,1)), real(I_pp_1d(1,2)),real(I_pp_1d(1,3)),...
                        real(I_pp_1d(2,1)), real(I_pp_1d(2,2)),real(I_pp_1d(2,3)),...
                        real(I_pp_1d(3,1)), real(I_pp_1d(3,2)),real(I_pp_1d(3,3)),...
                        real(I_pm_1d(1,1)), real(I_pm_1d(1,2)),real(I_pm_1d(1,3)),...
                        real(I_pm_1d(2,1)), real(I_pm_1d(2,2)),real(I_pm_1d(2,3)),...
                        real(I_pm_1d(3,1)), real(I_pm_1d(3,2)),real(I_pm_1d(3,3)));  

                %---- compute intensity and polarisation in each domain
		I_pp=I_pp.+I_pp_1d*popul(jj);
		I_pm=I_pm.+I_pm_1d*popul(jj);
		jj=jj+1;
        endfor

	%---- calculate final polarisation 
	polar=(I_pp.-I_pm)./(I_pp.+I_pm);

        fprintf(stdout,'\naverage for       %+3.2f %+3.2f %+3.2f      %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f         %+3.2f %+3.2f %+3.2f      %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f         %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f\n\n',...
        Q_rlu_print(1),Q_rlu_print(2),Q_rlu_print(3),...
        polar(1,1),polar(1,2),polar(1,3),polar(2,1),polar(2,2),polar(2,3),polar(3,1),polar(3,2),polar(3,3),...
        real(I_pp(1,1)), real(I_pp(1,2)),real(I_pp(1,3)),...
        real(I_pp(2,1)), real(I_pp(2,2)),real(I_pp(2,3)),...
        real(I_pp(3,1)), real(I_pp(3,2)),real(I_pp(3,3)),...
        real(I_pm(1,1)), real(I_pm(1,2)),real(I_pm(1,3)),...
        real(I_pm(2,1)), real(I_pm(2,2)),real(I_pm(2,3)),...
        real(I_pm(3,1)), real(I_pm(3,2)),real(I_pm(3,3)));  
        fprintf(matrix,'\naverage for       %+3.2f %+3.2f %+3.2f      %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f       %+3.2f %+3.2f %+3.2f      %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f         %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f\n\n',...
        Q_rlu_print(1),Q_rlu_print(2),Q_rlu_print(3),...
        polar(1,1),polar(1,2),polar(1,3),polar(2,1),polar(2,2),polar(2,3),polar(3,1),polar(3,2),polar(3,3),...
        real(I_pp(1,1)), real(I_pp(1,2)),real(I_pp(1,3)),...
        real(I_pp(2,1)), real(I_pp(2,2)),real(I_pp(2,3)),...
        real(I_pp(3,1)), real(I_pp(3,2)),real(I_pp(3,3)),...
        real(I_pm(1,1)), real(I_pm(1,2)),real(I_pm(1,3)),...
        real(I_pm(2,1)), real(I_pm(2,2)),real(I_pm(2,3)),...
        real(I_pm(3,1)), real(I_pm(3,2)),real(I_pm(3,3)));  
endfor

jj=1;
for k=1:domains_cell{1}:ll-1
	fprintf(stdout,"                                                               Final Polarisation                                                      Intensity (- :--> +)                                                     Intensity (- :--> -)\n");
	fprintf(stdout,"Type     Popul. |   Q (rlu)            | -XX    -XY   -XZ   |  -YX   -YY   -YZ     |  -ZX   -ZY   -ZZ         | -XX    -XY   -XZ   |  -YX   -YY   -YZ     |  -ZX   -ZY   -ZZ         | -X-X   -X-Y  -X-Z  |  -Y-X  -Y-Y  -Y-Z    |  -Z-X  -Z-Y   -Z-Z   |\n");
	fprintf(matrix,"                                                               Final Polarisation                                                      Intensity (- :--> +)                                                     Intensity (- :--> -)\n");
	fprintf(matrix,"Type     Popul. |   Q (rlu)            | -XX    -XY   -XZ   |  -YX   -YY   -YZ     |  -ZX   -ZY   -ZZ         | -XX    -XY   -XZ   |  -YX   -YY   -YZ     |  -ZX   -ZY   -ZZ         | -X-X   -X-Y  -X-Z  |  -Y-X  -Y-Y  -Y-Z    |  -Z-X  -Z-Y   -Z-Z   |\n");
%---- Pi along -x
        I_mp=zeros(3,3);
        I_mm=zeros(3,3);
        polar=zeros(3,3);
	popul_nz=0;
        for ii=1:domains_cell{1}
                H=hkl_all(1,jj);
                K=hkl_all(2,jj);
                L=hkl_all(3,jj);
                %----- Q vector in cartesian coordinates (1/A)
                Q=[H;K;L]';   
                Q_rlu=[hkl_all_rlu(1,jj);hkl_all_rlu(2,jj);hkl_all_rlu(3,jj)]';
		if (ii == 1) Q_rlu_print=Q_rlu; endif;
                %---- second vector in scattering plane
                %---- first vector is given by Q
                A1=AA1_all(1:3,jj);
                [I_mp_1d,I_mm_1d,polar_1d]=...
                feval("onedomain",P_1,P_2,Q,Q_rlu,A1',flag_k0,flag_nmi,spins_cart_cell,dtype(jj),'pi_down',propvec_cell_rlu,det_r(jj),nucstrucfac(jj));

                fprintf(stdout,'%s       %+3.2f     %+3.2f %+3.2f %+3.2f      %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f         %+3.2f %+3.2f %+3.2f      %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f         %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f\n',...
                        dtype(jj),popul(jj),...
                        Q_rlu(1),Q_rlu(2),Q_rlu(3),...
                        polar_1d(1,1),polar_1d(1,2),polar_1d(1,3),polar_1d(2,1),polar_1d(2,2),polar_1d(2,3),polar_1d(3,1),polar_1d(3,2),polar_1d(3,3),...
                        real(I_mp_1d(1,1)), real(I_mp_1d(1,2)),real(I_mp_1d(1,3)),...
                        real(I_mp_1d(2,1)), real(I_mp_1d(2,2)),real(I_mp_1d(2,3)),...
                        real(I_mp_1d(3,1)), real(I_mp_1d(3,2)),real(I_mp_1d(3,3)),...
                        real(I_mm_1d(1,1)), real(I_mm_1d(1,2)),real(I_mm_1d(1,3)),...
                        real(I_mm_1d(2,1)), real(I_mm_1d(2,2)),real(I_mm_1d(2,3)),...
                        real(I_mm_1d(3,1)), real(I_mm_1d(3,2)),real(I_mm_1d(3,3)));  
                fprintf(matrix,'%s       %+3.2f     %+3.2f %+3.2f %+3.2f      %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f         %+3.2f %+3.2f %+3.2f      %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f         %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f\n',...
                        dtype(jj),popul(jj),...
                        Q_rlu(1),Q_rlu(2),Q_rlu(3),...
                        polar_1d(1,1),polar_1d(1,2),polar_1d(1,3),polar_1d(2,1),polar_1d(2,2),polar_1d(2,3),polar_1d(3,1),polar_1d(3,2),polar_1d(3,3),...
                        real(I_mp_1d(1,1)), real(I_mp_1d(1,2)),real(I_mp_1d(1,3)),...
                        real(I_mp_1d(2,1)), real(I_mp_1d(2,2)),real(I_mp_1d(2,3)),...
                        real(I_mp_1d(3,1)), real(I_mp_1d(3,2)),real(I_mp_1d(3,3)),...
                        real(I_mm_1d(1,1)), real(I_mm_1d(1,2)),real(I_mm_1d(1,3)),...
                        real(I_mm_1d(2,1)), real(I_mm_1d(2,2)),real(I_mm_1d(2,3)),...
                        real(I_mm_1d(3,1)), real(I_mm_1d(3,2)),real(I_mm_1d(3,3))); 

                I_mp=I_mp.+I_mp_1d.*popul(jj);
                I_mm=I_mm.+I_mm_1d.*popul(jj);
		jj++;	
        endfor


	%---- calculate final polarisation 
	polar=(I_mp.-I_mm)./(I_mp.+I_mm);

        fprintf(stdout,'\naverage for       %+3.2f %+3.2f %+3.2f      %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f         %+3.2f %+3.2f %+3.2f      %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f         %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f\n\n',...
        Q_rlu_print(1),Q_rlu_print(2),Q_rlu_print(3),...
        polar(1,1),polar(1,2),polar(1,3),polar(2,1),polar(2,2),polar(2,3),polar(3,1),polar(3,2),polar(3,3),...
        real(I_mp(1,1)), real(I_mp(1,2)),real(I_mp(1,3)),...
        real(I_mp(2,1)), real(I_mp(2,2)),real(I_mp(2,3)),...
        real(I_mp(3,1)), real(I_mp(3,2)),real(I_mp(3,3)),...
        real(I_mm(1,1)), real(I_mm(1,2)),real(I_mm(1,3)),...
        real(I_mm(2,1)), real(I_mm(2,2)),real(I_mm(2,3)),...
        real(I_mm(3,1)), real(I_mm(3,2)),real(I_mm(3,3)));  
        fprintf(matrix,'\naverage for       %+3.2f %+3.2f %+3.2f      %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f         %+3.2f %+3.2f %+3.2f      %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f         %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f\n\n',...
        Q_rlu_print(1),Q_rlu_print(2),Q_rlu_print(3),...
        polar(1,1),polar(1,2),polar(1,3),polar(2,1),polar(2,2),polar(2,3),polar(3,1),polar(3,2),polar(3,3),...
        real(I_mp(1,1)), real(I_mp(1,2)),real(I_mp(1,3)),...
        real(I_mp(2,1)), real(I_mp(2,2)),real(I_mp(2,3)),...
        real(I_mp(3,1)), real(I_mp(3,2)),real(I_mp(3,3)),...
        real(I_mm(1,1)), real(I_mm(1,2)),real(I_mm(1,3)),...
        real(I_mm(2,1)), real(I_mm(2,2)),real(I_mm(2,3)),...
        real(I_mm(3,1)), real(I_mm(3,2)),real(I_mm(3,3)));  
endfor
fprintf(stdout,'\nelapsed time %+3.2f (s)\n',toc);
fprintf(matrix,'\nelapsed time %+3.2f (s)\n',toc);
fclose(matrix);

elseif (cont == 6)
%---- fit nuclear intensities/ structure factors

%----- get cell parameters etc... from readnuc
       [genpar,as,bs,cs,aa,bb,cc,ion_cell,spgsym_cell,spgtrans_cell,twins,singlecrys,nparameters,dnparameters,...
       nfit_type,simplexnuc_options,lmfitnuc_options,nsim_ann_par,draw_nuc_opt]=readnuc;

%----- prepare hkl,intensities,errors
	%--- hkl-list
	nr_hkl=singlecrys{2}+2;
	hkl_crys_rlu=[[singlecrys{1,3:nr_hkl}].hhkl_rlu];
	hkl_crys_rlu_meas=reshape(hkl_crys_rlu,3,columns(hkl_crys_rlu)/3);
	hkl_crys_AA=[[singlecrys{1,3:nr_hkl}].hhkl_AA];
	hkl_crys_AA_meas=reshape(hkl_crys_AA,3,columns(hkl_crys_AA)/3);

	hkl_crys_rlu_inp=hkl_crys_rlu_meas;
	hkl_crys_AA_inp=hkl_crys_AA_meas;
	%--- add twins
        [Mcryst2cart,Mrec2cart]=lat2cart(as,bs,cs,aa,bb,cc);
        for i=1:columns(twins)
                tw=(Mrec2cart*twins{1,i}.rot*Mrec2cart^-1)*hkl_crys_rlu_meas;
                hkl_crys_rlu_inp=[hkl_crys_rlu_inp tw];
                tw=(Mrec2cart*twins{1,i}.rot*Mrec2cart^-1)*hkl_crys_AA_meas;
                hkl_crys_AA_inp=[hkl_crys_AA_inp tw];
        endfor
	%--- intensity list
	int_crys_inp=[[singlecrys{1,3:nr_hkl}].f2obs];
	%--- error list
	err_crys_inp=1./[[singlecrys{1,3:nr_hkl}].f2sd];
	%--- switch intensity <-> structure factor
	intfsqr=singlecrys{1}.intfsqr;

	nlat.as=as;nlat.bs=bs;nlat.cs=cs;nlat.aa=aa;nlat.bb=bb;nlat.cc=cc;	

%----- Rotation matrices to transform anisotropic B-factors tensors
        %---- loop over symmetry elements and
        %---- store rotation matrix to transform Uij tensor
        if (ion_cell{columns(ion_cell)}.Btype == 1)
          Rmat = zeros(3,3,spgsym_cell{1});
          Rmatinv = zeros(3,3,spgsym_cell{1});
          for j = 1:spgsym_cell{1}
            trs.X = spgsym_cell{j+1}.x;
            trs.Y = spgsym_cell{j+1}.y;
            trs.Z = spgsym_cell{j+1}.z;
            [R, trans] = sym2mat(trs);  %---- transformation matrix
            Rmat(1:3,1:3,j) = R;
            Rmatinv(1:3,1:3,j) = R^-1;
          endfor
        else
          Rmat = zeros(3,3);
          Rmatinv = zeros(3,3);
        endif

	switch nfit_type
%---- fit parameters options
	case 0
	%---- Levenberg-Marquard algorithm
		%---- Function to minimize
		F=@nucfacfit2;
		%---- numerical derivatives
		dFdp=@dfdp;
		%---- additional convergence options
		minstep=zeros(size(nparameters));
		maxstep=Inf*ones(size(nparameters));	
		options=[minstep, maxstep];
		%---- verbose
		verbose=lmfitnuc_options.verbose;
        	%---- exit options
        	stol=lmfitnuc_options.stol;
        	niter=lmfitnuc_options.niter;

        	%---- print out fitting options
        	disp( '* Levenberg-Marquard algorithm' );
        	disp( '* fit options:');
        	fprintf(stdout,'--> Exit if tol. <= %f\n',stol);
        	fprintf(stdout,'--> Max. number of iterations: %i\n', niter);
        	if (verbose == 0)
                	disp('--> fit progress not shown');
        	else
                	disp('--> fit progess shown');
        	endif

		tic
		[f,p,deltap,pout,dpout,ChiSq,R_Bragg,kvg,iter,corp,covp,covr,stdresid,Z,r2]= ...
	      	       leasqr(int_crys_inp, err_crys_inp, nparameters, dnparameters,F,...
                       stol, niter, verbose, dFdp,options,...
                       hkl_crys_rlu_meas',hkl_crys_AA_inp',...
                       genpar,nlat,intfsqr,...
                       ion_cell,spgsym_cell,spgtrans_cell,columns(twins)+1,Rmat,Rmatinv);
		toc
	case 1
	%---- Nelder-Mead algorithm
		stol=simplexnuc_options.stol;   
		niter=simplexnuc_options.niter;
		fval=simplexnuc_options.fval;
		sides=simplexnuc_options.sides;
		verbose=simplexnuc_options.verbose;
		%---- stop fit options, -1 is to minimise cost function (see
		%---- nmsmax.m)
		stopit = [stol niter fval sides verbose -1];

		%---- print out fitting options
		disp( '* Nelder-Mead algorithm' );
		disp( '* fit options:');
		fprintf(stdout,'--> Exit if simplex size <= %f\n',stol);
		fprintf(stdout,'--> Max. number of iterations: %i\n', niter); 
		fprintf(stdout,'--> Exit if ChiSq <= %f\n', fval);
		if (sides == 0)
        		disp('--> uses regular simplex');
		else
		        disp('--> uses right-angled simplex');
		endif
		if (verbose == 0)
		        disp('--> fit progress not shown');
		else
		        disp('--> fit progess shown');
		endif
		disp('* results saved in file savit');
		savit='SAVE SAVIT x fmax nf';
	%---- start fit
		[p, deltap, f, ChiSq, nf, pout, dpout, R_Bragg]=nmsmax("chi",nparameters,dnparameters,stopit,'savit',...
                                              "nucfacfit2",int_crys_inp,err_crys_inp,...
                                               hkl_crys_rlu_meas',hkl_crys_AA_inp',...
                                               genpar,nlat,intfsqr,...
                                               ion_cell,spgsym_cell,spgtrans_cell,columns(twins)+1,Rmat,Rmatinv);

	case 2
       %---- fit with simulated annealing
                %---- control 
                ub=nparameters.*(1.+dnparameters.*sign(nparameters));
                lb=nparameters.*(1.-dnparameters.*sign(nparameters));
                nt=nsim_ann_par.nt;
                ns=nsim_ann_par.ns;
                rt=nsim_ann_par.rt;
                maxevals=nsim_ann_par.maexevals;
                neps=nsim_ann_par.neps;
                functol=nsim_ann_par.functol;
                paramtol=nsim_ann_par.paramtol;
                verbosity=nsim_ann_par.verbosity;
                minarg = 1; # position of parameters in control (counter starts at 0)
                control = {lb, ub, nt, ns, rt, maxevals, neps, functol, paramtol,verbosity, 1 };

                %---- print options
                disp( '* Simulated-Annealing algorithm' );
                disp( '* options:');
                fprintf(stdout,'---> # of evaluations before T reduction:  %i\n',nt);
                fprintf(stdout,'---> # of iterations between bounds adjustments: %i\n',ns);
                fprintf(stdout,'---> T reduction factor: %f\n',rt);
                fprintf(stdout,'---> maximum iterations: %i\n',maxevals);
                fprintf(stdout,'---> function tolerance: %f\n',functol);
                fprintf(stdout,'---> parameters tolerance: %f\n',paramtol);
                fprintf(stdout,'---> verbosity: %i\n',verbosity);
                fflush(stdout);

                %unfixed=length(find(dp~=0));

                %---- argument of function 
                args={nparameters,dnparameters,"nucfacfit2",int_crys_inp,err_crys_inp,...
		      hkl_crys_rlu_meas',hkl_crys_AA_inp',...
                      genpar,nlat,intfsqr,...
                      ion_cell,spgsym_cell,spgtrans_cell,columns(twins)+1,Rmat,Rmatinv};

                %---- start simulated annealing
                [pout, ChiSq, convergence] = samin("chi", args, control);

	endswitch
	if ((nfit_type == 0) ||(nfit_type == 1))

		f=f';

		%---- print out fit results
		disp( '* Least Square Estimates of Parameters');
		disp('      value      sigma(+/-)');
		fprintf(stdout,'%i:  %f   %f\n',[ [1:rows(p)]' real(p) real(deltap) ]');
		disp('* Chi squared');
		disp(abs(ChiSq));
		disp('* R_Bragg');
		disp(R_Bragg);
                %----- save fcalc^2/fobs^2 to file
                %----- for fltk to plot
                [fid]=fopen('fltkplot','wt');
                  output=[f',int_crys_inp']';
                  fprintf(fid,"%f %f\n",output);
                fclose(fid);
		%----- Plot Obs. vs. Calc. Intensities
		clf;
		hold on;
		plot([0:1.1*max(f)],[0:1.1*max(f)],"-r");	
		plot(int_crys_inp',f',"o","markersize",10);	
		axis([0,1.1*max(f),0,1.1*max(f)],"square");
		xlabel("observed");ylabel("calculated");title("fit result");
		hold off;

		%----- Print out results to sreen and to file (crysfit.nuc)
		%write results to file
                fprintf(stdout,"FIT TERMINATED");
		fprintf('\n\n* Results of fit written in file crysfit.nuc\n');
		[fid]=fopen('crysfit.nuc','wt');
		fprintf(fid,'\n* R_Bragg = %+f\n', R_Bragg);
		fprintf(fid,'\n* ChiSq = %+f\n', ChiSq);
		%---- Print out parameters
		fprintf(fid,'\n Fitted Parameters:\n');
        	%---- parameters 1-->10
        	fprintf(fid,'!  N1      N2      N3       N4      N5      N6      N7      N8      N9      N10\n');
        	fprintf(fid,'%+8.4f ',real(pout(1:10))');
        	fprintf(fid,'\n');
	        %parameters shifts 1-->10
	        fprintf(fid,'!steps dN1 to DN10\n');
	        fprintf(fid,'%+8.4f ',dnparameters(1:10)');
	        fprintf(fid,'\n');

        	%---- parameters 11-->20
        	fprintf(fid,'!  N11      N12      N13      N14      N15      N16       N17     N18      N19      N20\n');
        	fprintf(fid,'%+8.4f ',real(pout(11:20))');
        	fprintf(fid,'\n');
	        %parameters shifts 11-->20
	        fprintf(fid,'!steps dN11 to DN20\n');
	        fprintf(fid,'%+8.4f ',dnparameters(11:20)');
	        fprintf(fid,'\n');

        	%---- parameters 21-->30
        	fprintf(fid,'!  N21      N22      N23      N24      N25      N26      N27      N28      N29      N30\n');
        	fprintf(fid,'%+8.4f ',real(pout(21:30))');
        	fprintf(fid,'\n');
	        %parameters shifts 21-->30
	        fprintf(fid,'!steps dN1 to DN30\n');
	        fprintf(fid,'%+8.4f ',dnparameters(21:30)');
	        fprintf(fid,'\n');

        	%---- parameters 31-->40
        	fprintf(fid,'!  N31      N32      N33      N34      N35      N36      N37      N38      N39      N40\n');
        	fprintf(fid,'%+8.4f ',real(pout(31:40))');
        	fprintf(fid,'\n');
	        %parameters shifts 31-->40
	        fprintf(fid,'!steps dN1 to DN40\n');
	        fprintf(fid,'%+8.4f ',dnparameters(31:40)');
	        fprintf(fid,'\n');

        	%---- parameters 41-->50
        	fprintf(fid,'!  N41      N42      N43      N44      N45      N46      N47      N48      N49      N50\n');
        	fprintf(fid,'%+8.4f ',real(pout(41:50))');
        	fprintf(fid,'\n');
	        %parameters shifts 41-->50
	        fprintf(fid,'!steps dN1 to DN50\n');
	        fprintf(fid,'%+8.4f ',dnparameters(41:50)');
	        fprintf(fid,'\n');

		fprintf(fid,'\n* Calculations :\n');	 
		fprintf(fid,'# H        K         L        Calc            Obs           StdErr          Diff\n');
		fprintf(fid,'%+2.3f  %+2.3f  %+2.3f    %+10.3f     %+10.3f    %+10.3f   %+10.3f\n',[hkl_crys_rlu_meas(1:3,:)',f',int_crys_inp',1./err_crys_inp',f'.-int_crys_inp']');
	
		fclose(fid);
	endif
       fflush(stdout);
elseif (cont == 7)
%---- fit magnetic integrated intensities with Levenberg-Marquard (0),
%---- Nelder-Mead (1) or Simulated annealing (2)

	[genpar,singlecrys,...
	as,bs,cs,aa,bb,cc,...
	param,dp,...
	flag_k0,flag_nmi,k0,moment_type,...
	fit_type,simplex_options,lmfit_options,sim_ann_par,...
	lat_cell,ion_cell,moment_cell,propvec_cell_rlu,propvec_cell_AA,spgsym_cell,spgtrans_cell,domains_cell,...	
	draw_mag_opt]=...
	crys_inp;


	nr_hkl=singlecrys{2}+2;
%---- hkl-list
	hkl_crys=[[singlecrys{1,3:nr_hkl}].hhkl_rlu];
	hkl_meas=reshape(hkl_crys,columns(hkl_crys)/3,3);

%---- intensity list
	int_crys_obs=[[singlecrys{1,3:nr_hkl}].f2obs];
%---- error list
	err_crys_obs=1./[[singlecrys{1,3:nr_hkl}].f2sd];
%---- intensities
	intfsqr=singlecrys{1}.intfsqr;

%----- get the list of magnetic moments in the unit cell in cartesian coordinates 
	[M_cell,spins_cart_cell]=moment(lat_cell,ion_cell,moment_cell,propvec_cell_AA,spgsym_cell,spgtrans_cell);
%---- needs lambda to calculate form factor
	lambda=genpar.lambda;
%---- prepare hkl-list with domains, calculate formfactor, Lorenz factor
	[QQ,fstar,formfactor,popul,llor]=hklprep(as,bs,cs,aa,bb,cc,lambda,singlecrys,domains_cell,spins_cart_cell,propvec_cell_rlu);

%----- switch for fit algorithm
	switch fit_type
		%---- Levenberg-Marquard
	case 0
	       %---- input parameters
	        pin=param;
	        %---- Function to minimize
	        F=@crys_int_fit2;
	        %---- numerical derivatives
	        dFdp=@dfdp;
	        %---- additional convergence options
	        minstep=zeros(size(pin));
	        maxstep=Inf*ones(size(pin));
	        options=[minstep, maxstep]; 
	        %---- if verbose 1 output of fit results at the end of leasqr.m
	        global verbose;
	        verbose = 2;   
	        %---- exit options
	        stol=lmfit_options.stol;
	        niter=lmfit_options.niter;
	
	        %---- print out fitting options
	        disp( '* Levenberg-Marquard algorithm' );
	        disp( '* fit options:');
	        fprintf(stdout,'--> Exit if tol. <= %f\n',stol);
	        fprintf(stdout,'--> Max. number of iterations: %i\n', niter);
	        if (verbose == 0)
	                disp('--> fit progress not shown');
	        else
	                disp('--> fit progess shown');
	        endif

	        tic

		[f,p,deltap,pout,dpout,ChiSq,R_Bragg,kvg,iter,corp,covp,covr,stdresid,Z,r2]=...
	              leasqr(int_crys_obs,err_crys_obs, pin, dp,F,...
                            stol,niter,verbose, dFdp,options,...  
	                    hkl_meas,QQ,lat_cell,ion_cell,spgsym_cell,spgtrans_cell,moment_cell,propvec_cell_AA,...
	                    fstar,flag_k0,formfactor,genpar,intfsqr,llor,domains_cell);

		%---- Nelder-Mead
	case 1
		%---- simplex fit parameters options
		stol=simplex_options.stol;
		niter=simplex_options.niter;
		fval=simplex_options.fval;
		sides=simplex_options.sides;
		verbose=simplex_options.verbose;
		stopit = [stol niter fval sides verbose -1];

	%---- print out fitting options
		disp( '* Nelder-Mead algorithm' );
		disp( '* fit options:');
		fprintf(stdout,'--> Exit if simplex size <= %f\n',stol);
		fprintf(stdout,'--> Max. number of iterations: %i\n', niter);
		fprintf(stdout,'--> Exit if ChiSq <= %f\n', fval);
		if (sides == 0)
		        disp('--> uses regular simplex');
		else
		        disp('--> uses right-angled simplex');
		endif
		if (verbose == 0)
		        disp('--> fit progress not shown');
		else
		        disp('--> fit progess shown');
		endif
		disp('* results saved in file savit');
		savit='SAVE SAVIT x fmax nf';
	%---- start fit
		[p, deltap, f, ChiSq,nf,pout, dpout, R_Bragg]=nmsmax("chi",param,dp,stopit,'savit',...
	                        "crys_int_fit2",int_crys_obs,err_crys_obs,...
	                	hkl_meas,QQ,lat_cell,ion_cell,spgsym_cell,spgtrans_cell,moment_cell,propvec_cell_AA,...
				fstar,flag_k0,formfactor,genpar,intfsqr,llor,domains_cell);

		%---- simulated annealing
	case 2
	%---- fit with simulated annealing
	        %---- control 
	        ub=param.*(1.+dp.*sign(param));
	        lb=param.*(1.-dp.*sign(param));
	        nt=sim_ann_par.nt;
	        ns=sim_ann_par.ns;
	        rt=sim_ann_par.rt;
	        maxevals=sim_ann_par.maexevals;
	        neps=sim_ann_par.neps;
	        functol=sim_ann_par.functol;
	        paramtol=sim_ann_par.paramtol;
	        verbosity=sim_ann_par.verbosity;
	        minarg = 1; # position of parameters in control (counter starts at 0)
	        control = { lb, ub, nt, ns, rt, maxevals, neps, functol, paramtol,verbosity, 1 };

		%---- print options
                disp( '* Simulated-Annealing algorithm' );
                disp( '* options:');
		fprintf(stdout,'---> # of evaluations before T reduction: %i\n',nt);
		fprintf(stdout,'---> # of iterations between bounds  adjustments: %i\n',ns);
		fprintf(stdout,'---> T reduction factor: %f\n',rt);
		fprintf(stdout,'---> maximum iterations: %i\n',maxevals);
		fprintf(stdout,'---> function tolerance: %f\n',functol);
		fprintf(stdout,'---> parameters tolerance: %f\n',paramtol);
		fprintf(stdout,'---> verbosity: %i\n',verbosity);
                fflush(stdout);

	        %unfixed=length(find(dp~=0));
	        %---- argument of function 
	        args={param,dp,"crys_int_fit2",int_crys_obs,err_crys_obs,...
		      hkl_meas,QQ,lat_cell,ion_cell,spgsym_cell,spgtrans_cell,moment_cell,propvec_cell_AA,...
	              fstar,flag_k0,formfactor,genpar,intfsqr,llor,domains_cell};

	        %---- start simulated annealing
	        [pout, ChiSq, convergence] = samin("chi", args, control);

	endswitch

	if ((fit_type == 0) ||(fit_type == 1))
		f=f';

%---- print out fit results
		disp( '* Least Square Estimates of Parameters');
		disp('      value      sigma(+/-)');
		fprintf(stdout,'%i:  %f   %f\n',[ [1:rows(p)]' real(p) real(deltap) ]');
		disp('* Chi squared');
		disp(abs(ChiSq));
		disp('* R_Bragg');
		disp(R_Bragg);
%---- print out observed/calculate fsqr
        	nr_hkl=singlecrys{2}+2;
        	hkl_crys_rlu=[[singlecrys{1,3:nr_hkl}].hhkl_rlu];
        	hkl_crys_rlu_inp=reshape(hkl_crys_rlu,3,columns(hkl_crys_rlu)/3);
        	int_crys_inp=[[singlecrys{1,3:nr_hkl}].f2obs];
        	err_crys_inp=1./[[singlecrys{1,3:nr_hkl}].f2sd];
        	intfsqr=singlecrys{1}.intfsqr;


		fprintf(stdout,' H        K         L        Calc             Obs              StdErr          Diff.\n');
		fprintf(stdout,'%+2.3f  %+2.3f  %+2.3f    %+10.3f       %+10.3f       %+10.3f       %+10.3f\n',[hkl_crys_rlu_inp(1:3,:)',f',int_crys_inp',1./err_crys_inp',f'.-int_crys_inp']');
		[fid]=fopen('crysfit.mag','wt');
		fprintf(fid,'* Least Squares Estimates of Parameters\n');
		fprintf(fid,"    %+3.2f  %+3.2f  %+3.2f  %+3.2f  %+3.2f  %+3.2f  %+3.2f  %+3.2f   %+3.2f   %+3.2f\n",pout);
		fprintf(fid,'* Estimates of Parameters Errors\n');
		fprintf(fid,"    %+3.2f  %+3.2f  %+3.2f  %+3.2f  %+3.2f  %+3.2f  %+3.2f  %+3.2f   %+3.2f   %+3.2f\n",dpout);


		fprintf(fid,'\n* R_Bragg = %+f\n', R_Bragg);
		fprintf(fid,'\n* ChiSq = %+f\n', abs(ChiSq));
		fprintf(fid,'\n* Calculations :\n');	 
		fprintf(fid,'# H        K         L        Calc             Obs              StdErr          Diff.\n');
		fprintf(fid,'%+2.3f  %+2.3f  %+2.3f    %+10.3f       %+10.3f       %+10.3f       %+10.3f\n',[hkl_crys_rlu_inp(1:3,:)',f',int_crys_inp',1./err_crys_inp',f'.-int_crys_inp']');
		fclose(fid);	
		fprintf('\n\n* Results of fit written in file crysfit.mag\n');

		clf;
		hold on;
		plot([0:1.1*max(f)],[0:1.1*max(f)],"-r");	
		axis([0,1.1*max(f),0,1.1*max(f)],"square");	
		plot(int_crys_inp',f',"o","markersize",10);	
		xlabel("observed");ylabel("calculated");title("fit result");
		hold off;
%data = [int_crys_inp' f'];
%save magplot data;

	else
		disp('* Chi squared');
		disp(ChiSq);	
	endif

elseif (cont == 8)

%---- import everything necessary to calculate and fit polarimetry data, see readmu.m
	[as,bs,cs,aa,bb,cc,param,dp,flag_k0,flag_nmi,kk0,hhkl_AA,hhkl_rlu,AA1,Pi,Pfo,dPfo,nr,P_1,P_2,moment_type,...
	fit_type,simplex_options,lmfit_options,sim_ann_par,...
	lat_cell,ion_cell,moment_cell,propvec_cell_rlu,propvec_cell_AA,spgsym_cell,spgtrans_cell,domains_cell,...
	draw_mag_opt]=readmu;
	%----  augment various lists because of domains
	ll=1;
	singlecrys=cell(1,nr/3);
	singlecrys{2}=nr/3;
	V_bloc=[];
	jj=1;
	for kk=1:3:nr
		singlecrys{1,jj+2}.hhkl_AA=hhkl_AA(kk,1:3);
		singlecrys{1,jj+2}.hhkl_rlu=hhkl_rlu(kk,1:3);
	       	for ii=1:domains_cell{1}
			%---- rotation matrices for domains are given in a cartesian coordinates in input file
	       	        hkl_all(1:3,ll)=domains_cell{ii+1}.rot*(hhkl_AA(kk,1:3))';
	       	        AA1_all(1:3,ll)=domains_cell{ii+1}.rot*AA1(kk,1:3)';
	       	        det_r=domains_cell{ii+1}.det;

			V1=hkl_all(1:3,ll);
			V2=AA1_all(1:3,ll);
			V3=cross(V1,V2);
			V2=cross(V3,V1); 
			if (dot(V1/norm(V1),V2/norm(V2)) == 1 )
			        error("\nredefine scattering plane\n");
			        return;
			endif
			V1=V1./norm(V1);
			V2=V2./norm(V2);
			V3=V3./norm(V3);
			V3=V3.*det_r;
			V=[V1';V2';V3'];
			V_bloc=blkdiag(V_bloc,V);

			if (flag_nmi ~= 0)
				%---- do not rotate hkl's for nuclear reflections				
				hkl_all_nuc(1:3,ll)=hhkl_AA(kk,1:3)';
			endif
			
	                ll++;
	        endfor
		jj=jj+1;
	endfor
	
	ll=1;
	Pi_all_cell=cell;
	for kk=1:3:nr-2
		for ii=1:domains_cell{1}
			Pi_all_cell{ll,1}=Pi(kk:kk+2,1:3);
			ll++;
		endfor
	endfor
	Pi_all=cell2mat(Pi_all_cell);
	%----- get the list of magnetic moments in the unit cell in cartesian coordinates
	[M_cell,spins_cart_cell]=moment(lat_cell,ion_cell,moment_cell,propvec_cell_AA,spgsym_cell,spgtrans_cell);

	%---- prepare hkl-list with domains, calculate formfactor, Lorenz factor
	lambda=1;  %---- used to calculate Lorenz factor; not needed anyway for polarisation matrices
	[QQ,fstar,formfactor,popul,llor]=hklprep(as,bs,cs,aa,bb,cc,lambda,singlecrys,domains_cell,...
                                                 spins_cart_cell,propvec_cell_rlu);


	%---- read atomic parameters if flag_nmi ~= 0
	if (flag_nmi ~= 0)
	  [genpar,as,bs,cs,aa,bb,cc,ion_nuc_cell,spgsym_nuc_cell,spgtrans_nuc_cell,twins,singlecrys,...
	   parameters,dparameters,...
	   nfit_type,simplexnuc_options,lmfitnuc_options,nsim_ann_par,...
	   draw_nuc_opt]=readnuc;
	   [nucstrucfac]=feval("nucfac",as,bs,cs,aa,bb,cc,ion_nuc_cell,spgsym_nuc_cell,spgtrans_nuc_cell,hkl_all_nuc');
        else
           nucstrucfac=0;
	endif


%---- start fit
	switch fit_type
	case 0
%---- Levenberg-Marquardt algorithm 
tic
		%---- input parameters
		pin=param;
		%---- Function to minimize
	        F=@polar_fit;
	        %---- numerical derivatives
	        dFdp=@dfdp;
	        %---- additional convergence options
	        minstep=zeros(size(pin));
	        maxstep=Inf*ones(size(pin));
	        options=[minstep, maxstep]; 
	        %---- if verbose 1 output of fit results at the end of leasqr.m
		verbose=lmfit_options.verbose;
	        %---- exit options
	        stol=lmfit_options.stol;
	        niter=lmfit_options.niter;
	
	        %---- print out fitting options
	        disp( '* Levenberg-Marquard algorithm' );
	        disp( '* fit options:');
	        fprintf(stdout,'--> Exit if tol. <= %f\n',stol);
	        fprintf(stdout,'--> Max. number of iterations: %i\n', niter);
	        if (verbose == 0)
	                disp('--> fit progress not shown');
	        else
	                disp('--> fit progess shown');
	        endif

tic
		err_Pfo=1./reshape(dPfo',1,3*nr);
		[f,p,deltap,pout,dpout,ChiSq,R_Bragg,kvg,iter,corp,covp,covr,stdresid,Z,r2]=...
	                    leasqr(reshape(Pfo',1,columns(Pfo)*rows(Pfo)),err_Pfo, pin, dp,F,...
                                   stol,niter,verbose, dFdp,options,... 
	                           reshape(Pi',3*nr,1),Pi_all,lat_cell,ion_cell,spgsym_cell,spgtrans_cell,propvec_cell_AA,moment_cell,...
	                           fstar,QQ,flag_k0,formfactor,...
	                           P_1,P_2,V_bloc,flag_nmi,domains_cell,nucstrucfac);

                %---- print out fit results
                disp( '* Least Square Estimates of Parameters');
                disp('      value      sigma(+/-)');
		fprintf(stdout,'%i:  %f   %f\n',[ [1:rows(p)]' real(p) real(deltap) ]');
                disp('* Chi squared');
                disp(abs(ChiSq));

toc
	%---- fitted polarisation matrices
        pol_fitted=(reshape(f,3,nr))';

        fprintf(stdout,"\n hkl,incident,  observed and calculated scattered polarisations\n");
	[fid_out]=fopen('mufit.out','wt');
        fprintf(fid_out,"\n hkl,incident,  observed and calculated scattered polarisations\n");
        pol_fitted=(reshape(f,3,nr))';
        dpfo=round(100.*dPfo);
        for k=1:nr
                fprintf(stdout,"%2.3f %2.3f %2.3f || %+2.2f %+2.2f %+2.2f|  %+2.2f (%2i)   %+2.2f (%2i)   %+2.2f (%2i) | %+2.2f %+2.2f %+2.2f\n",...
                hhkl_rlu(k,1),hhkl_rlu(k,2),hhkl_rlu(k,3),Pi(k,1),Pi(k,2),Pi(k,3),Pfo(k,1),dpfo(k,1),Pfo(k,2),dpfo(k,2),Pfo(k,3),dpfo(k,3),pol_fitted(k,1),pol_fitted(k,2),pol_fitted(k,3));
                fprintf(fid_out,"%2.3f %2.3f %2.3f || %+2.2f %+2.2f %+2.2f|  %+2.2f (%2i)   %+2.2f (%2i)   %+2.2f (%2i) | %+2.2f %+2.2f %+2.2f\n",...
                hhkl_rlu(k,1),hhkl_rlu(k,2),hhkl_rlu(k,3),Pi(k,1),Pi(k,2),Pi(k,3),Pfo(k,1),dpfo(k,1),Pfo(k,2),dpfo(k,2),Pfo(k,3),dpfo(k,3),pol_fitted(k,1),pol_fitted(k,2),pol_fitted(k,3));
        endfor
	fclose(fid_out);


	case 1
	%----- fit the polarisation matrices with Nelder-Mead method (simplex)

	%---- fit with nmsmax

	%---- parameters options
	stol=simplex_options.stol;
	niter=simplex_options.niter;
	fval=simplex_options.fval;
	sides=simplex_options.sides;
	verbose=simplex_options.verbose;
	stopit = [stol niter fval sides verbose -1];
	
	%---- print out fitting options
	disp( '* Nelder-Mead algorithm' );
	disp( '* fit options:');
	fprintf(stdout,'--> Exit if simplex size <= %f\n',stol);
	fprintf(stdout,'--> Max. number of iterations: %i\n', niter);
	fprintf(stdout,'--> Exit if ChiSq <= %f\n', fval);
	if (sides == 0)
		disp('--> uses regular simplex');
	else
		disp('--> uses right-angled simplex');
	endif
	if (verbose == 0)
		disp('--> fit progress not shown');
	else
		disp('--> fit progess shown');
	endif
	disp('* results saved in file savit');
	savit='SAVE SAVIT x fmax nf';
	
	%---- start fit
	err_Pfo=1./reshape(dPfo',1,3*nr);
	[p, deltap, f, ChiSq,nf,pout,dpout]=nmsmax("chi",param,dp,stopit,'savit',...
	                          "polar_fit",reshape(Pfo',1,columns(Pfo)*rows(Pfo)),err_Pfo,...
	                          reshape(Pi',3*nr,1),Pi_all,lat_cell,ion_cell,spgsym_cell,spgtrans_cell,propvec_cell_AA,moment_cell,...
	                          fstar,QQ,flag_k0,formfactor,...
	                          P_1,P_2,V_bloc,flag_nmi,domains_cell,nucstrucfac);
	%---- print out fit results
	disp( '* Least Square Estimates of Parameters');
	disp('      value      sigma(+/-)');
		fprintf(stdout,'%i:  %f   %f\n',[ [1:rows(p)]' real(p) real(deltap)]');
	disp('* Chi squared');
	disp(abs(ChiSq));
	
	%---- print out observed and calculated matrices
        pol_fitted=(reshape(f,3,nr))';
        fprintf(stdout,"\n hkl,incident,  observed and calculated scattered polarisations\n");
        [fid_out]=fopen('mufit.out','wt');
        fprintf(fid_out,"\n hkl,incident,  observed and calculated scattered polarisations\n");
        pol_fitted=(reshape(f,3,nr))';
	dpfo=round(100.*dPfo);
        for k=1:nr
                fprintf(stdout,"%2.3f %2.3f %2.3f || %+2.2f %+2.2f %+2.2f|  %+2.2f (%2i)   %+2.2f (%2i)   %+2.2f (%2i) | %+2.2f %+2.2f %+2.2f\n",...
                hhkl_rlu(k,1),hhkl_rlu(k,2),hhkl_rlu(k,3),Pi(k,1),Pi(k,2),Pi(k,3),Pfo(k,1),dpfo(k,1),Pfo(k,2),dpfo(k,2),Pfo(k,3),dpfo(k,3),pol_fitted(k,1),pol_fitted(k,2),pol_fitted(k,3));
                fprintf(fid_out,"%2.3f %2.3f %2.3f || %+2.2f %+2.2f %+2.2f|  %+2.2f (%2i)   %+2.2f (%2i)   %+2.2f (%2i) | %+2.2f %+2.2f %+2.2f\n",...
                hhkl_rlu(k,1),hhkl_rlu(k,2),hhkl_rlu(k,3),Pi(k,1),Pi(k,2),Pi(k,3),Pfo(k,1),dpfo(k,1),Pfo(k,2),dpfo(k,2),Pfo(k,3),dpfo(k,3),pol_fitted(k,1),pol_fitted(k,2),pol_fitted(k,3));
        endfor
        fclose(fid_out);


	case 2

	%---- end of what will be taken out

	%----- Simulated Annealing
        %---- control 
        ub=param.*(1.+dp.*sign(param));
        lb=param.*(1.-dp.*sign(param));
        nt=sim_ann_par.nt;
        ns=sim_ann_par.ns;
        rt=sim_ann_par.rt;
        maxevals=sim_ann_par.maexevals;
        neps=sim_ann_par.neps;
        functol=sim_ann_par.functol;
        paramtol=sim_ann_par.paramtol;
        verbosity=sim_ann_par.verbosity;
        minarg = 1; # position of parameters in control (counter starts at 0)
        control = { lb, ub, nt, ns, rt, maxevals, neps, functol, paramtol,verbosity, 1 };

        %---- print options
        disp( '* Simulated-Annealing algorithm' );
        disp( '* options:');
        fprintf(stdout,'---> # of evaluations before T reduction: %i\n',nt);
        fprintf(stdout,'---> # of iterations between bounds adjustments: %i\n',ns);   
        fprintf(stdout,'---> T reduction factor: %f\n',rt);
        fprintf(stdout,'---> maximum iterations: %i\n',maxevals);
        fprintf(stdout,'---> function tolerance: %f\n',functol);
        fprintf(stdout,'---> parameters tolerance: %f\n',paramtol);
        fprintf(stdout,'---> verbosity: %i\n',verbosity);
        fflush(stdout);
        %---- start fit
        err_Pfo=1./reshape(dPfo',1,3*nr);
        %---- argument of function 
	args={param,dp,"polar_fit",reshape(Pfo',1,columns(Pfo)*rows(Pfo)),err_Pfo,...
              reshape(Pi',3*nr,1),Pi_all,lat_cell,ion_cell,spgsym_cell,spgtrans_cell,propvec_cell_AA,moment_cell,...
              fstar,QQ,flag_k0,formfactor,...
               P_1,P_2,V_bloc,flag_nmi,domains_cell,nucstrucfac};

        %---- start simulated annealing
        [pout, ChiSq, convergence] = samin("chi", args, control);
	endswitch

	if ((fit_type == 0) ||(fit_type == 1))
	%---- write matrices to Latexfile
		disp('* Observed and calculated matrices witten in ''mufit.tex''');
		[fid_tex]=fopen('mufit.tex','wt');
		cmnd='\';
		fprintf(fid_tex,'%sdocumentclass[10pt,a4paper]{article}\n',cmnd);
		fprintf(fid_tex,'%susepackage{ifpdf}\n',cmnd);
		fprintf(fid_tex,'%susepackage[utf8]{inputenc}\n',cmnd);
		fprintf(fid_tex,'%stitle{Polarisation Matrices}\n',cmnd);
		fprintf(fid_tex,'%sbegin{document}\n',cmnd);
		fprintf(fid_tex,'%smaketitle\n',cmnd);
		fprintf(fid_tex,'%sbegin{center}\n',cmnd);
		fprintf(fid_tex,'%sbegin{tabular}{ccc|ccc|ccc|ccc}\n',cmnd);
		ret='\\';
        	for k=1:nr
                	fprintf(fid_tex,"%2.2f & %2.2f & %2.2f & %1i & %1i & %1i  & %2.2f (%i)   & %2.2f (%i)   & %2.2f (%i) & %2.2f & %2.2f & %2.2f %s\n",...
                                         hhkl_rlu(k,1),hhkl_rlu(k,2),hhkl_rlu(k,3),...
                                         Pi(k,1),Pi(k,2),Pi(k,3),Pfo(k,1),...
                                         dpfo(k,1),Pfo(k,2),dpfo(k,2),...
                                         Pfo(k,3),dpfo(k,3),pol_fitted(k,1),pol_fitted(k,2),pol_fitted(k,3),ret);
        	endfor
		fprintf(fid_tex,'%send{tabular}\n',cmnd);
		fprintf(fid_tex,'%send{center}\n',cmnd);
		fprintf(fid_tex,'%send{document}\n',cmnd);
		fclose(fid_tex);
	endif

elseif (cont == 9)
%----- draw magnetic structure

%---- import everything necessary to calculate and fit polarimetry data, see readmu.m
	s=input("crys.inp (0) or mag.inp (1)?: ");
	switch s
	  case 0
	    [genpar,singlecrys,...
            as,bs,cs,aa,bb,cc,...
            param,dp,...
            flag_k0,flag_nmi,k0,moment_type,...
            fit_type,simplex_options,lmfit_options,sim_ann_par,...
            lat_cell,ion_cell,moment_cell,propvec_cell_rlu,propvec_cell_AA,spgsym_cell,spgtrans_cell,domains_cell,...
            draw_mag_opt]=...
            crys_inp;
	  case 1
	    [as,bs,cs,aa,bb,cc,param,dp,flag_k0,flag_nmi,kk0,hhkl_AA,hhkl_rlu,AA1,Pi,Pfo,dPfo,nr,P_1,P_2,moment_type,...
	    fit_type,simplex_options,lmfit_options,sim_ann_par,...
	    lat_cell,ion_cell,moment_cell,propvec_cell_rlu,propvec_cell_AA,spgsym_cell,spgtrans_cell,domains_cell,...
	    draw_mag_opt]=readmu;
	  otherwise
 	    error("unknow option\n");
	endswitch
	%---- transformation to cartesian coordinates
        [Mcryst2cart,Mrec2cart]=lat2cart(as,bs,cs,aa,bb,cc);

	%---- magnetic moments components
	[M_cell,spins_cart_cell]=moment(lat_cell,ion_cell,moment_cell,propvec_cell_AA,spgsym_cell,spgtrans_cell);

        %----- remove ions with coordinates > lattice parameters, if draw unitcell true

        k=2;

	lrec.a = lat_cell.ll(1)*Mcryst2cart*[1;0;0];
        if (draw_mag_opt{1,end}.unitcell == 1)
         M_cell2{1,1}=0;
         for i=2:M_cell{1,1}+1

  	   %----- atomic positions already in cartesian coordinates
           %----- but not lattice constants
           if (Mcryst2cart^-1*M_cell{i,1}.mat' >= -[0.001 0.001 0.001]' & ...
	       Mcryst2cart^-1*M_cell{i,1}.mat' <= [lat_cell.ll].+0.001)
             M_cell2{k,1}=M_cell{i,1};
             k++;
	   endif 
         endfor
        M_cell2{1,1}=k-2;
        M_cell=M_cell2;
        endif

	%----- draw magnetic structure
	lat.as=as;
	lat.bs=bs;
	lat.cs=cs;
	nr_spins=moment_cell{1,1};
	polar_cart=moment_cell{1,nr_spins+2}.type;
	mt_draw_mag(Mcryst2cart,lat, M_cell, polar_cart, draw_mag_opt);

elseif (cont == 10)
%---- draw crystal structure

%----- get cell parameters etc... from readnuc
	[genpar,as,bs,cs,aa,bb,cc,ion_cell,spgsym_cell,spgtrans_cell,twins,singlecrys,nparameters,dnparameters,...
	nfit_type,simplexnuc_options,lmfitnuc_options,nsim_ann_par,draw_nuc_opt]=readnuc;

%-----  vrml
%-----  draw crystal structure
         [Mcryst2cart,Mrec2cart]=lat2cart(as,bs,cs,aa,bb,cc);
	switch draw_nuc_opt{1,end}.style
	 case 0 
%----    ball and stick
	 %----- get atomic distances
	 [atoms_cell,atdists_cell]=mt_atom_dist(ion_cell, spgtrans_cell, spgsym_cell, draw_nuc_opt);
	 %----- create vrml-script
	 lat.as=as;lat.bs=bs;lat.cs=cs;lat.aa=aa;lat.bb=bb;lat.cc=cc;
	 mt_draw_nuc(lat, atoms_cell, atdists_cell, Mcryst2cart, draw_nuc_opt);

	 case 1  
%-----   ellispoids
	 %----- get atomic distances
	 [atoms_cell,atdists_cell]=mt_atom_dist(ion_cell, spgtrans_cell, spgsym_cell, draw_nuc_opt);
	 a_c=as*[1; 0; 0];b_c=bs*[0; 1; 0];c_c=cs*[0; 0; 1];
	 M=Mcryst2cart;
	 A=[...
	  M(:,1)'*a_c M(:,1)'*b_c M(:,1)'*c_c;...
	  M(:,2)'*a_c M(:,2)'*b_c M(:,2)'*c_c;...
	  M(:,3)'*a_c M(:,3)'*b_c M(:,3)'*c_c];
	 ra=norm(Mrec2cart(:,1));
	 rb=norm(Mrec2cart(:,2));
	 rc=norm(Mrec2cart(:,3));
	 B=[ra 0 0; 0 rb 0; 0 0 rc]/2/pi;
	 D=A*B; %----Acta Cryst. A52, 770 (1996)

%----- Real space lattice vectors in cartesian coordinates     
	 for k=1:columns(atoms_cell)
	   %----- transform to cartesian axis
	   UC=D*atoms_cell{1,k}.radius*transpose(D);
	   atoms_cell{1,k}.radius=UC;
	 endfor
	 %----- create vrml-script
	 lat.as=as;lat.bs=bs;lat.cs=cs;lat.aa=aa;lat.bb=bb;lat.cc=cc;
	 mt_draw_nuc(lat, atoms_cell, atdists_cell, Mcryst2cart, draw_nuc_opt);

	endswitch

elseif (cont == 11)
%----- space group
	mt_space_group;

elseif (cont == 12)
%----- get cell parameters etc... from readnuc
       [genpar,as,bs,cs,aa,bb,cc,ion_cell,spgsym_cell,spgtrans_cell,twins,singlecrys,nparameters,dnparameters,...
       nfit_type,simplexnuc_options,lmfitnuc_options,nsim_ann_par,draw_nuc_opt]=readnuc;
       as = ion_cell{2}.as;
       bs = ion_cell{2}.bs;
       cs = ion_cell{2}.cs;
       aa = ion_cell{2}.aa;
       bb = ion_cell{2}.bb;
       cc = ion_cell{2}.cc;
       [gij]=metric(as,bs,cs,aa,bb,cc);
       gsij = inv(gij);
       astar = sqrt(gsij(1,1));
       bstar = sqrt(gsij(2,2));
       cstar = sqrt(gsij(3,3));
       cosgs = gsij(2,1)/astar/bstar;
       cosbs = gsij(3,1)/astar/cstar;
       cosas = gsij(3,2)/bstar/cstar;
       for i = 1:ion_cell{1}
         ion = ion_cell{i+2}.ion;
         beta11 = ion_cell{i+2}.u11*astar*astar;
         beta22 = ion_cell{i+2}.u22*bstar*bstar;
         beta33 = ion_cell{i+2}.u33*cstar*cstar;
         beta12 = ion_cell{i+2}.u12*astar*bstar*cosgs;
         beta13 = ion_cell{i+2}.u13*astar*cstar*cosbs;
         beta23 = ion_cell{i+2}.u23*bstar*cstar*cosas;
         beta21 = beta12;
         beta31 = beta13;
         beta32 = beta23;
         Beta_mat = 2*pi^2*[beta11 beta12 beta13; beta21 beta22 beta23; beta31 beta32 beta33];
#{
         A = [as    bs*cosd(cc)   cs*cosd(bb);
              0     bs*sind(bb)   -cs*sind(bb)*cosas
              0     0             1/cstar];
         UC = (2*pi^2)^-1*transpose(A)*Beta_mat*A;
#} 
         Ueq = trace(Beta_mat*gij)/3/2/pi^2;
         Beq = 8*pi^2*Ueq;
         fprintf("ion: %s, B_eq = %3.2f ang^2,  <|u|> = %3.2f ang\n",ion, Beq, sqrt(2*Beq));
       endfor
    
elseif (cont == 13)
%----- flipping ratios
        [genpar,as,bs,cs,aa,bb,cc,ion_cell,spgsym_cell,spgtrans_cell,p,singlecrys]=...
        read_flipping_ratio_input();
        [N, NN, M, MM, NMI, FR]=calcflippingratios(genpar,as,bs,cs,aa,bb,cc,ion_cell,spgsym_cell,spgtrans_cell,p,singlecrys);
        %--- NN  nuclear structure factor
        %--- MM  magnetic structure factor
        %--- NMI nuclear-magnetic interaction part
        %--- FR   flipping ratio
        output=zeros(singlecrys{1,2},13);
        for i =1:singlecrys{1,2}
#{
           %compute phase of nuclear structure factor
           if (abs(real(N(i))) < 1e-10) N(i)=i*imag(N(i)); endif
           if (abs(imag(N(i))) < 1e-10) N(i)=real(N(i)); endif
           if (real(N(i))>0) phi_n = atan(imag(N(i))/real(N(i))); endif
           if ((real(N(i))<0) && (imag(N(i)) >= 0)) phi_n = atan(imag(N(i))/real(N(i)))+pi; endif
           if ((real(N(i))<0) && (imag(N(i)) < 0)) phi_n = atan(imag(N(i))/real(N(i)))-pi; endif
           if ((real(N(i))==0) && (imag(N(i)) > 0)) phi_n = pi/2; endif
           if ((real(N(i))==0) && (imag(N(i)) < 0)) phi_n = -pi/2; endif
           if ((real(N(i))==0) && (imag(N(i)) == 0)) phi_n = 0; endif
           %compute phase of magnetic structure factor
           if (abs(real(M(i))) < 1e-10) M(i)=i*imag(M(i)); endif
           if (abs(imag(M(i))) < 1e-10) M(i)=real(M(i)); endif
           if (real(M(i))>0) phi_m = atan(imag(N(i))/real(N(i))); endif
           if ((real(M(i))<0) && (imag(M(i)) >= 0)) phi_m = atan(imag(M(i))/real(M(i)))+pi; endif
           if ((real(M(i))<0) && (imag(M(i)) < 0)) phi_m = atan(imag(M(i))/real(M(i)))-pi; endif
           if ((real(M(i))==0) && (imag(M(i)) > 0)) phi_m = pi/2; endif
           if ((real(M(i))==0) && (imag(M(i)) < 0)) phi_m = -pi/2; endif
           if ((real(M(i))==0) && (imag(M(i)) == 0)) phi_m = 0; endif
#}
           %compute phase of nuclear structure factor
           R=sqrt(real(N(i))^2+imag(N(i)^2));
           if (R < 0.000001)
              phi_n = 0;
           else
              phi_n=acos(real(N(i))/R);
              if (imag(N(i)) < 0) phi_n = -phi_n; endif
           endif
           %compute phase of magnetic structure factor
           R=sqrt(real(M(i))^2+imag(M(i)^2));
           if (R < 0.000001)
              phi_m = 0;
           else
              phi_m=acos(real(M(i))/R);
              if (imag(M(i)) < 0) phi_m = -phi_m; endif
           endif
           output(i,:)=[singlecrys{1,i+2}.hhkl_rlu real(N(i)) imag(N(i)) phi_n NN(i) real(M(i))  imag(M(i))  phi_m  MM(i) NMI(i) FR(i)];
        endfor
        [sortedcol,key]=sort(output(:,13),'descend');
        so=output(key,:);
        [fid]=fopen('flipratio.out','w');
	fprintf(fid,' H    K    L    Re(F_N)   Im(F_N)   phi_n     F_N^2      Re(F_M)     Im(F_M)     phi_m     F_M^2      NMI    R  \n' )
	fprintf(fid,'----------------------------------------------------------------------------------------------------------------\n') 
        for i=1:rows(so)
           fprintf(fid,"%+i   %+i   %+i   %+6.3f    %+6.3f    %+6.3f    %+6.3f    %+6.3f    %+6.3f    %+6.3f    %+6.3f     %+6.3f    %+6.3f\n",...
           so(i,1),so(i,2),so(i,3),so(i,4),so(i,5),so(i,6),so(i,7),so(i,8),so(i,9),so(i,10),so(i,11),so(i,12),so(i,13));
        endfor
        fclose(fid);
        disp('*---  Simulation written in file ''flipratio.out''');
elseif (cont == 14)
%----- doc
	manual;

elseif (cont == 15)
	exit=0;
	return;
endif

%---- write file with output
if (cont == 7 | cont == 8 )
        [Mcryst2cart,Mrec2cart]=feval("lat2cart",as,bs,cs,aa,bb,cc);
	fprintf(stdout, '* Input file written in file ''mag.new''\n');
        [file_new]=fopen('mag.new','wt');

        fprintf(file_new,'!lattice\n');
        fprintf(file_new,'%+2.4f  %+2.4f  %+2.4f  %+3.2f  %+3.2f  %+3.2f\n',as,bs,cs,aa,bb,cc);
        fprintf(file_new,'!#atoms\n');
        fprintf(file_new,'%i\n',ion_cell{1});
        fprintf(file_new,'!AT   x         y        z        B         occ\n');
        for i=1:ion_cell{1}
                fprintf(file_new,'%s  %+1.5f  %+1.5f  %+1.5f  %+1.5f %+1.5f\n',...
                        ion_cell{i+1}.ion,...
                        ion_cell{i+1}.x,ion_cell{i+1}.y,ion_cell{i+1}.z,...
                        ion_cell{i+1}.B,ion_cell{i+1}.occ);
        endfor
        fprintf(file_new,'! input switch 0: components along axis RMx,RMy,RMz,Imx,Imy,Imz,phase; 1: polar coordinates Rm,Rhi,Rtheta,Im,Iphi,Iptheta,phase\n');
        fprintf(file_new,'%i\n',moment_type);
        if (moment_type == 0)
                fprintf(file_new,'!       F(Q)       Rmx           Rmy            Rmz               Imx            Imy             Imz          phase (units of 2*pi)\n');
                fprintf(file_new,'%i\n',moment_cell{1});
                for i=1:moment_cell{1}
                        fprintf(file_new,'%s   %s',moment_cell{i+1}.ion, moment_cell{i+1}.fq);
			fprintf(file_new,'   %+2.3f         %+2.3f         %+2.3f',moment_cell{i+1}.mx,moment_cell{i+1}.my, moment_cell{i+1}.mz);
			fprintf(file_new,'   %+2.3f         %+2.3f         %+2.3f',moment_cell{i+1}.imx, moment_cell{i+1}.imy, moment_cell{i+1}.imz);
                        fprintf(file_new,'   %+2.3f\n',moment_cell{i+1}.phase);
                endfor
       else
                fprintf(file_new,'!       F(Q)    Rm       Rphi     Rth       Im    Iphi     Ith     phase (units of 2*pi)\n');
                fprintf(file_new,'%i\n',moment_cell{1});
                for i=1:moment_cell{1}
                        fprintf(file_new,'%s   %s',moment_cell{i+1}.ion,moment_cell{i+1}.fq);
			fprintf(file_new,'   %+3.3f         %+3.3f         %+3.3f',moment_cell{i+1}.rm,moment_cell{i+1}.rphi,moment_cell{i+1}.rtheta);
			fprintf(file_new,'   %+3.3f         %+3.3f         %+3.3f',moment_cell{i+1}.im,moment_cell{i+1}.iphi,moment_cell{i+1}.itheta);
			fprintf(file_new,'   %+2.3f\n',moment_cell{i+1}.phase);
                endfor
        endif   
        fprintf(file_new,'!# symmetry operations in space group and symmetry elements\n');
        fprintf(file_new,'%i\n',spgsym_cell{1});
	[min,max]=size(spgsym_cell);
        for i=min:max-1
                fprintf(file_new,'   %s   %s   %s',spgsym_cell{1,i+1}.x,spgsym_cell{1,i+1}.y,spgsym_cell{1,i+1}.z)
                fprintf(file_new,'   %s   %s   %s',spgsym_cell{1,i+1}.rmx,spgsym_cell{1,i+1}.rmy,spgsym_cell{1,i+1}.rmz);
                fprintf(file_new,'   %s   %s   %s',spgsym_cell{1,i+1}.imx,spgsym_cell{1,i+1}.imy,spgsym_cell{1,i+1}.imz);
		fprintf(file_new,'   %2.3f\n',spgsym_cell{1,i+1}.phase);
        endfor
        fprintf(file_new,'!# number of translations and translations\n');
        fprintf(file_new,'%i\n',spgtrans_cell{1});
        for i=1:spgtrans_cell{1}
                fprintf(file_new,'   %i   %i   %i\n',spgtrans_cell{i+1}.x./as,spgtrans_cell{i+1}.y./bs,spgtrans_cell{i+1}.z./cs);
        endfor
        fprintf(file_new,'!# propagation vector\n'); 
        for i=1:propvec_cell_rlu{1}
                fprintf(file_new,'%s   %+1.3f   %+1.3f   %+1.3f\n',propvec_cell_rlu{i+1}.label,...
                        [propvec_cell_rlu{i+1}.kx;propvec_cell_rlu{i+1}.ky;propvec_cell_rlu{i+1}.kz]);
        endfor
        fprintf(file_new,'!#spin domains, populations and parameters\n');
        fprintf(file_new,'%i\n',domains_cell{1});
        fprintf(file_new,'!population and rotation matrices for domains\n');
        for i=1:domains_cell{1}
                fprintf(file_new,'%1.2f', domains_cell{i+1}.pop);
		fprintf(file_new,'    %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f     %+3.2f %+3.2f %+3.2f', domains_cell{i+1}.rot);
		fprintf(file_new,'    %s\n',domains_cell{i+1}.type);
        endfor
        %parameters 1-->10
        fprintf(file_new,'!c1     c2     c3     c4     c5     c6     c7     c8     c9     c10\n');
        fprintf(file_new,'%+6.2f ',pout(1:10)');
        fprintf(file_new,'\n');
        %parameters shifts 1-->10
        fprintf(file_new,'!steps dC1 to DC30\n');
        fprintf(file_new,'%+6.2f ',dp(1:10)');
        fprintf(file_new,'\n');

        fprintf(file_new,'!c11    c12    c13    c14    c15    c16    c17    c18    c19    c20\n');
        fprintf(file_new,'%+6.2f ',pout(11:20)');
        fprintf(file_new,'\n');
        fprintf(file_new,'!steps dC11 to DC20\n');
        fprintf(file_new,'%+6.2f ',dp(11:20)');
        fprintf(file_new,'\n');

        fprintf(file_new,'!c21    c22    c23    c24    c25    c26    c27    c28     c29    c30\n');
        fprintf(file_new,'%+6.2f ',pout(21:30)');
        fprintf(file_new,'\n');
        fprintf(file_new,'!steps dC1 to DC30\n');
        fprintf(file_new,'%+6.2f ',dp(21:30)');   
        fprintf(file_new,'\n');

        fprintf(file_new,'!D1     D2     D3     D4     D5     D6     D7     D8      D9     D10\n');
        fprintf(file_new,'%+6.2f ',pout(31:40)');
        fprintf(file_new,'\n');
        fprintf(file_new,'!steps dD1 to DD30\n');
        fprintf(file_new,'%+6.2f ',dp(31:40)');   
        fprintf(file_new,'\n');


        fprintf(file_new,'! flag for calculation k0 + -k0 (1); flag for pure magnetic reflection (0)\n');
        fprintf(file_new,'%i   %i\n',flag_k0,flag_nmi);
	if (cont == 8)
        	fprintf(file_new,'! bender efficiencies 0.5 < P_1, P_2 < 1\n');
        	fprintf(file_new,'%1.2f   %1.2f\n',P_1,P_2);
	endif
        fclose(file_new);
endif

clear all;
endfunction
