## Copyright (C) 2007,2008,2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##
## usage: mufit
##

# -*- texinfo -*-
## @deftypefn{Function File}  ncs
## (@var{H},@var{K},@var{L},@var{A1},@var{P0_x},@var{P0_y},@var{P0_z},@var{mag_inter_vec},@var{flag_nmi},@var{det_r})
## @deftypefnx{Function File} {[@var{P_f},@var{cs_n}]}= ncs 
## (@var{H},@var{K},@var{L},@var{A1},@var{P0_x},@var{P0_y},@var{P0_z},@var{mag_inter_vec},@var{flag_nmi},@var{det_r})
## @*Calculate neutron cross section and polarisation matrices with the
## Blume's equations
##
##@*input:
##	@*1.- [H,K,L]
##	@*2.- @var{A1} second vector in scattering plane (A^-1, cartesian coordinates)
##	@*3.- incident polarisation [P0_x;P0_y;P0_z]
##	@*4.- @var{mag_inter_vec}: the magnetic interaction vector in crystal
##	coordinate system 
##	@*5.- @var{flag_nmi}:flag for mixed magnetic/nuclear reflections 
##      @*6.- @var{det_r}: determinant of the domain operation
##
##@*output:
##      @*1.- @var{P_F}: the scattered polarisation
##      @*2.- @var{ncs}: the neutron cross-section

## @end deftypefn


function [P_f,cs_n]=ncs(Q,A1,P0_x,P0_y,P0_z,mag_inter_vec,flag_nmi,det_r,nucstrucfac);

P_f=zeros(1,3);

%---- polarisation of incident neutron beam in polarimeter frame
P_0=[P0_x;P0_y;P0_z];

V1=Q';
%---- component of vector in scattering plane in cartesian coordinates (1/A)
V2=A1';

if (dot(V1/norm(V1),V2/norm(V2)) == 1 )
	error("\nredefine scattering plane\n");
	return;
endif

%----- Form unit vectors V1, V2, V3 in scattering plane
V3=cross(V1,V2);
V2=cross(V3,V1);
V3=V3/sqrt(sum(V3.*V3));
V2=V2/sqrt(sum(V2.*V2));
V1=V1/sqrt(sum(V1.*V1));
V3=V3.*det_r;

%---- coordinates of HKL in V1, V2, V3 frame
%---- transformation matrix
U=[V1';V2';V3'];

%---- rotate magnetic interaction vector into MuPAD system
magstruc=U*mag_inter_vec;

%---- calculate cross-section terms
%---- 1) sigma nuclear
sigma_n=nucstrucfac.*conj(nucstrucfac);

%---- 2)sigma magnetic 
sigma_m=magstruc'*magstruc;

%---- 3)nuclear magnetic interference
sigma_imn=P_0'*(nucstrucfac.*conj(magstruc).+conj(nucstrucfac).*magstruc);

%---- 4)chiral term
sigma_ch=P_0'*I*cross(conj(magstruc),magstruc);

%---- cross-section
cs_n=sigma_n.+sigma_m.+sigma_imn.+sigma_ch;
if (cs_n == 0)
	cs_n = 10^-8;
endif

%---- calculates final polarisation vector according to Blume
%---- 1) nuclear
P_i_n=P_0.*sigma_n;

%---- 2) magnetic
P_i_m=-P_0.*sigma_m+conj(magstruc).*(P_0'*magstruc).+(magstruc'*P_0).*magstruc;

%---- 3) nuclear-magnetic interference term
nmi=conj(nucstrucfac).*magstruc.-conj(magstruc).*nucstrucfac;
P_i_imn=nucstrucfac.*conj(magstruc)+conj(nucstrucfac).*magstruc+I.*cross(nmi,P_0);

%---- 4) chiral part
P_i_ch=-I.*cross(conj(magstruc),magstruc);

%---- complete neutron cross-section
P_i=P_i_n.+P_i_m.+P_i_imn.+P_i_ch;
P_i(find(abs(P_i)<0.000001))=0;

%---- polarisation vector after scattering
P_f=real(P_i./cs_n);

endfunction;

