## Copyright (C) 2007,2008,2009 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##

## -*- texinfo -*-
## @deftypefn{Function File}  polar_fit2 (@var{p},@var{varargin})
## @deftypefnx{Function File} {[@var{polar_final_row},@var{param}]} = polar_fit2 (@var{p},@var{varargin})
## @*Used to fit the polarization matrices
## @*Calculate polarized cross-section and calls __polmat.cc that actually
## performs the calculation of the polarization matrices
##
## @*input:
##       @*1.- @var{p} should be a vector containing the parameters
##       @*2.- @var{varargin} is a cell array containing the other variables
##
## @*output:
##       @*1.- @var{polar_final_row}: polarisation matrices
##       @*2.- @var{param}: parameter list  
## @end deftypefn

function [polar_final_row,param]=polar_fit2(p,varargin)

%---- extract parameters
param=p;
Pi=varargin{1};
Pi_all=varargin{2};
lat_cell=varargin{3};
ion_cell=varargin{4};
spgsym_cell=varargin{5};
spgtrans_cell=varargin{6};
propvec_cell_AA=varargin{7};
moment_cell=varargin{8};
fstar=varargin{9};
QQ=varargin{10};
flag_k0=varargin{11};
formfactor=varargin{12};
P_1=varargin{13};
P_2=varargin{14};
V_bloc=varargin{15};
flag_nmi=varargin{16};
domains_cell=varargin{17};
nucstrucfac=varargin{18};


%---- parameters of domain populations
D1=abs(param(31));
D2=abs(param(32));
D3=abs(param(33));
D4=abs(param(34));
D5=abs(param(35));
D6=abs(param(36));
D7=abs(param(37));
D8=abs(param(38));
D9=abs(param(39));
D10=abs(param(40));

%---- normalize domain population
sum_dom=D1+D2+D3+D4+D5+D6+D7+D8+D9+D10;
D1=D1/sum_dom; 
D2=D2/sum_dom; 
D3=D3/sum_dom; 
D4=D4/sum_dom; 
D5=D5/sum_dom;
D6=D6/sum_dom;
D7=D7/sum_dom;
D8=D8/sum_dom;
D9=D9/sum_dom;
D10=D10/sum_dom;

param(31)=D1; 
param(32)=D2; 
param(33)=D3; 
param(34)=D4; 
param(35)=D5;
param(36)=D6;
param(37)=D7;
param(38)=D8; 
param(39)=D9;
param(40)=D10;

%----- get the list of magnetic moments in the unit cell in cartesian coordinates
[spins_cart_cell]=moment_fit(lat_cell,ion_cell,spgsym_cell,spgtrans_cell,propvec_cell_AA,moment_cell,param);

%----- get magnetic structure factor for the hkl-list with domains
[mag_inter_vec,fsqr,mag_vec_cart]=magfac2(spins_cart_cell,fstar,QQ,flag_k0,formfactor);

%---- reverse sign of magnetic interaction vector if 180deg domain
tidom=[];
for i=1:domains_cell{1}
   if domains_cell{i+1}.type == 'T'
       %--- indices of T-type domains
       tidom=[tidom, i];
   endif
endfor
%---- reverese sign for these if 'T' domains exist
if !isempty(tidom)
    for i=tidom
        mag_inter_vec(1:i:end,:)=-mag_inter_vec(1:i:end,:);
    endfor
endif

%----- rotate magnetic interaction vector into polarimeter xyz-frame
magstruc=V_bloc*reshape(transpose(mag_inter_vec),rows(mag_inter_vec)*columns(mag_inter_vec),1);

%----- double elements to allow vectorization
magstruc=transpose(reshape(magstruc,3,rows(magstruc)/3));
%----- xyz components
magstruc_double1=repmat(vec(repmat(transpose(magstruc(:,1)),2,1)),1,2);
magstruc_double2=repmat(vec(repmat(transpose(magstruc(:,2)),2,1)),1,2);
magstruc_double3=repmat(vec(repmat(transpose(magstruc(:,3)),2,1)),1,2);

%---- Pauli Matrices
sigx=[0 1; 1 0];
sigy=[0 -I; I 0];
sigz=[1 0; 0 -1];

%---- copy Pauli Matrices to allow vectorization
nr_copy=rows(magstruc);
sigx_rep=repmat(sigx,nr_copy,1);
sigy_rep=repmat(sigy,nr_copy,1);
sigz_rep=repmat(sigz,nr_copy,1);

%---- Magnetic cross-section
cs_mag=magstruc_double1.*sigx_rep.+magstruc_double2.*sigy_rep.+magstruc_double3.*sigz_rep;

%---- nuclear part of total cross-section
if (flag_nmi == 0)
	poln_cs=cs_mag;
else
      %---- 'double' size of nuclear structure factors vector to take into account spin
      %---- up and down incoming polarisation
      C=[]; 
      l=1;
      for i=1:columns(nucstrucfac)
          for k=1:2
              C(l)=nucstrucfac(i);
              l=l+1;
          endfor
      endfor
      C=transpose(C);
      C=[C C];
      b=repmat(eye(2,2),columns(nucstrucfac),1);
      cs_nuc=b.*C;
      %---- sum of both magnetic and nuclear contributions
      poln_cs=cs_nuc+cs_mag;
endif

%--- calculate intensities and polaristaion matrices for each domain and hkl
[I_pp_1d_list,I_pm_1d_list,polar_1d_list]=__polmat(poln_cs,P_1,P_2,Pi_all);
%--- make a list
I_pp_1d_list={I_pp_1d_list};
I_pm_1d_list={I_pm_1d_list};
polar_1d_list={polar_1d_list};

%---- get domain popuations
ndom=domains_cell{1};
for k=1:ndom
	popul(k)=eval(domains_cell{k+1}.par_pop);
endfor
%---- average intensities over domain populations
k=1;
ll=1;
for i=1:columns(popul):columns(polar_1d_list)
        I_pp_mat=zeros(3,3);
        I_pm_mat=zeros(3,3);
	for j=1:domains_cell{1}
	        I_pp_mat=I_pp_mat.+popul(j).*[I_pp_1d_list{ll}];
		I_pm_mat=I_pm_mat.+popul(j).*[I_pm_1d_list{ll}];
		ll=ll+1;
	endfor
        M = I_pp_mat+I_pm_mat;
        [a,b] = find(M == 0);
        I_pp_mat(a,b) = 10e-10;
        I_pm_mat(a,b) = 10e-10;
	polar_av{k,1}=(I_pp_mat-I_pm_mat)./(I_pp_mat+I_pm_mat);
	k=k+1;
endfor
%---- calculated polarisation matrices averaged over domains for all hkl 
polar_final=cell2mat(polar_av);

%----- reshape before sending back result
nc=columns(polar_final);
nr=rows(polar_final);
k=1;
polar_final_row=zeros(1,nr*nc);
for i=1:nr
	for j=1:nc
		polar_final_row(k)=polar_final(i,j);
		k++;
	endfor
endfor

endfunction
