## Copyright (C) 2012 Bertrand Roessli <bertrand.roessli@psi.ch>
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##

function CalcAngles()
%
% Calculate instrument angles
% 2theta, omega, chi, phi for HKL reflection 
% according to Busing-Levy
% variables to be defined below:
%       wavelength: lambda
%       lattice parameters: a,b,c
%       angles: aa,bb,cc
%       first reflection h1 =[h,k,l] and observed angles (chi1,ph1,omega1)
%       second reflection (not parallel) h2 and observed angles
%

%compute angles for reflection HKL
%bisecting geometry
HKL = [0; 1; 0];

%wavelength
lambda = 0.5;

%lattice parameters and angles
a = 5.96; b = 5.96; c = 8.37; aa = 90; bb = 90; cc = 90;

% two observed reflections
h1 = [1; 0; 0]; h2 = [0; 1; 0];

%at angles
%chi1 = 180; phi1 = -72.84; chi2 = 180; phi2 = -162.84;
chi1 = 0; phi1 = 0; chi2 = 0; phi2 = 0;
%bisecting geometry
omega1 =  -87.595;
omega2 =  -177.60;

%start calculations
% 1.- metric cobvariant and contravairant tensors
[G,Gstar] = metric(a,b,c,aa,bb,cc);

% 2.- calculate UB matrix
% omega, chi, phi
u1 = [omega1; chi1; phi1];
u2 = [omega2; chi2; phi2];

[UBMat] = UBcalc(a,b,c,aa,bb,cc,h1,h2,u1,u2,lambda)
save -ascii UBMat UBMat;

%theta and 2theta
dhkl = HKL'*Gstar*HKL;
dhkl = 1/sqrt(dhkl);

if (lambda/2/dhkl > 1) error("too large scattering angle\n"); endif;
theta = asind(lambda/2/dhkl);
theta2 = 2*theta;

hphi = UBMat * HKL;
phi = atan2(hphi(2),hphi(1))/pi*180;
chi = atan2(hphi(3),sqrt(hphi(1)^2+hphi(2)^2))/pi*180;

fprintf(stdout, "*wavelength: %3.4f AA\n",lambda);
fprintf(stdout, "*lattice parameters: a = %3.4f, b = %3.4f, c = %3.4f\n",a,b,c);
fprintf(stdout, "*lattice angles: aa = %3.4f, bb = %3.4f, cc = %3.4f\n", aa,bb,cc); 
fprintf(stdout, "*HKL = [ %3.4f, %3.4f, %3.4f]\n",HKL);
fprintf(stdout, "*Calculated angles:\n\t2theta = %3.4f\n\ttheta = %3.4f\n\tchi = %3.4f\n\tphi = %3.4f\n",...
                          theta2, theta, chi, phi);

%
%3.- Normal Beam geoemetry
%
fprintf(stdout,"*Normal Beam geometry\n")

UB = UBMat; %[0.016169 0.011969 0.063195; -.000545 0.083377 -0.009117; -0.162051 0.000945 0.006321]
hkl = [0; 1; 0];
lambda = 0.5;
hphi = UB * hkl;
[gamma,om,nu,status] = z1mToNormalBeam(lambda,hphi)

endfunction
