## Copyright (C) 2012 Bertrand Roessli <bertrand.roessli@psi.ch>          
##
##
## MuFit is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation;
## either version 2, or (at your option) any later version.
##
## Mufit is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE.  See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public
## License along with Octave; see the file COPYING.  If not,
## write to the Free Software Foundation, Inc., 51 Franklin Street,
## Fifth Floor, Boston, MA 02110-1301, USA.
##

function [gamma,om,nu,status] = z1mToNormalBeam(lambda, z1m)
%
% Normal beam geometry
% calculate omega, gamma (2theta detector) and nu (tilting angle of detector)
% from UB matrix
% taken from fourlib.c by M. Koennecke
%
% input: 
%       lambda : wavelength
%       z1m    : UB * HKL
%
%

  RD = 57.30; %conversion radian to degrees

  [status, dm theta] = calcTheta(lambda, z1m);
  if (!status) 
    gamma = 0;
    om = 0;
    nu = 0;
    status = 0;
    return;
  endif

  a = sqrt(z1m(1) * z1m(1) + z1m(2) * z1m(2));
  sint = sin(theta / RD);
  b = 2. * sint * sint / (lambda * a);
  if (b >= 1.)
    gamma =0;
    om = 0;
    nu = 0;
    status = 0;
    return;
  endif

  a = -atan2(z1m(2), -z1m(1));
  b = -asin(b);
  om = a + b;
  omdeg = om * RD;
  dum = zeros(3,3);
  [dum] = PhiMat(om);
  znew = dum*z1m;
  if (znew(1) < 0) 
    om = om - 2. * atan2(-znew(1), -znew(2));
    omdeg = om * RD;
  endif

  b = (sign(180., omdeg) + omdeg) / 360.;
  b = sign(1, b) * floor(abs(b));
  omdeg = omdeg - 360. * b;
  nu = asin(lambda * z1m(3));
  gamma = acos(cos(2. * (theta / RD)) / cos(nu));
  om = omdeg;   
  nu = nu * RD;
  gamma = gamma * RD;
  status  = 1;
endfunction


function [status, d, theta] = calcTheta(lambda, z1)
%----------------------------------------------------------------------
% calculate d-spacing and theta from  z1
% taken from fourlib.c written by M. Koennecke
%-----------------------------------------------------------------------*/

  dstar = sqrt(z1(1)*z1(1) + z1(2)*z1(2) + z1(3)*z1(3));
  if (dstar < .0001) 
    d = .0;
    theta = 0;
    status = 0;
    return;
  endif

  d = 1. / dstar;
  sintheta = lambda * dstar / 2.;
  if (abs(sintheta) > 1.0) 
    d = .0;
    theta = .0; 
    return;
  endif
  theta = asin(sintheta) * 180 / pi;
  status = 1;
endfunction

function [a] = sign(b,a)
  if (b >= .0)    
    a = abs(a);
  else
    a =  -abs(a); 
  endif
endfunction
